VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "TactileGrid"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private cAxis As Collection ' the nodes along the diagonal
Private buffer As String
Private Shift As Integer ' indicates a special shift key has been pressed
Private cFromNode As CNode
Private cToNode As CNode
Private cInformation As String ' the current information about the vertex pressed
Private Const GRID_SIZE As Integer = 7 ' the height/width of the grid
Private cStartIndex As Integer 'we add nodes from the current level to the axis: if cStartIndex > 1 then
    'we start at cStartIndex, not 1. (This counts from 1)
Private cAllNodes As Collection ' holds all the nodes in the level that might be added to cAxis
Private cLockedNodes() As Boolean ' holds locked nodes, those that don't move (This might as well count from 1)

Public Property Get axis() As Collection

    On Error Resume Next

    Set axis = cAxis
End Property

Public Sub GenerateAxis(currentNode As CNode)

    On Error Resume Next

    'populate cAxis with the current hierarchy level's nodes
    Dim nodeIterator As CNode
    Dim i As Integer
    
    'clear the cLockedNodes array
    ReDim cLockedNodes(GRID_SIZE)
    Set cAxis = New Collection
    Set cAllNodes = New Collection
    If currentNode.parent Is Nothing Then
        'current node is top node
        Call cAllNodes.add(currentNode)
    Else
        'current node is not top node: axis should be populated with siblings
        'add all the level nodes to cAllNodes
        For Each nodeIterator In currentNode.parent.childNodes
            Call cAllNodes.add(nodeIterator)
        Next nodeIterator
    End If
    cStartIndex = 1 'start at first node
    Call CreateAxis
End Sub

Public Function ShiftAxis(direction As Integer) As Boolean

    On Error Resume Next

    'shifts the axis in the direction given by integer: return true if moved
    Dim oldIndex As Integer
    
    'note old index
    oldIndex = cStartIndex
    'calculate the new start position
    cStartIndex = cStartIndex + direction
    'make sure the position is valid
    If cStartIndex < 1 Then cStartIndex = 1
    If cStartIndex > cAllNodes.Count - GRID_SIZE Then cStartIndex = oldIndex
    'find out if we actually moved
    If oldIndex = cStartIndex Then
        'we didn't move!
        ShiftAxis = False
    Else
        'we moved
        ShiftAxis = True
        'now generate the new axis
        Call CreateAxis
    End If
End Function

Private Sub CreateAxis()

    On Error Resume Next

'populates the cAxis collection from the cAllNodes collection, starting at cStartIndex
    Dim i As Integer
    Dim lastIndex As Integer
    Dim currentAxis(GRID_SIZE) As CNode 'holds the current node
    
    'store the current nodes
    For i = 1 To cAxis.Count
        Set currentAxis(i) = cAxis.item(i)
    Next i
    'calculate the nodes to show
    lastIndex = cStartIndex + GRID_SIZE - 1
    If lastIndex > cAllNodes.Count Then lastIndex = cAllNodes.Count
    Set cAxis = New Collection
    For i = cStartIndex To lastIndex
        'work out if this is a locked node (shows up even when we scroll)
        If cLockedNodes(i - cStartIndex + 1) Then
            'locked node: use node from currentAxis array
            Call cAxis.add(currentAxis(i - cStartIndex + 1))
        Else
            'not locked node: use node from all nodes collection
            Call cAxis.add(cAllNodes.item(i))
        End If
    Next i
End Sub

Public Function ToggleLockNode(index As Integer) As Boolean

    On Error Resume Next

'locks or unlocks a node in the axis so it doesn't move when you scroll.
    Debug.Print "Lock requested: " & index
    If index < 1 Or index > cAxis.Count Then
        'out of range
        ToggleLockNode = False
    Else
        'okay, got a node
        cLockedNodes(index) = Not cLockedNodes(index)
        Debug.Print "Locked: " & index
        Call CreateAxis
        ToggleLockNode = True
    End If
End Function

Public Sub BufferClear()

    On Error Resume Next

    buffer = Empty
    cInformation = Empty
End Sub

Public Sub BufferAdd(chars As Long)

    On Error Resume Next

    'record in a buffer what keys have been pressed, and therefore what the user wants to do
    'Debug.Print "Adding [" & Chr(chars) & "]"
    buffer = buffer & Chr(chars)
End Sub

Public Function BufferExecute(entire As Collection) As Boolean

    On Error Resume Next

    'act on the buffer contents
    'These can be one of two things:
    '1 coordinates, which will be assumed to be "X,Y", where x=left-right x and y = top-down coord
    '2 A shift indicator: first character is s, second is a number indicating shift status
    'We must also check for whether shift is non-zero, which changes our selection
    'returns true if a shift action was performed, false if not: therefore, to work out if the button
    'pressed did anything, we can check the return value: if true, it changed shift. If false, it may
    'have called GridPress; we can check the length of .information to find out if there was any input,
    'and play an error sound if none
    '3 a command: first character is C, second is a number indicating command, from list:
    'C1 = clear all locked nodes
    'Debug.Print "buffer:[" & buffer & "]"
    If Shift = 0 Then
        'unshifted: we can either (1) get a shift value or (2) get some plain vanilla coords
        If Left(buffer, 1) = "S" Then
            Shift = Asc(Right(buffer, 1)) - 48
            'Debug.Print "Shift " & shift & " Activated"
            BufferExecute = True
        ElseIf Left(buffer, 1) = "C" Then
            'command
            Call ExecuteCommand(Asc(Right(buffer, 1)) - 48)
            BufferExecute = True
        Else
            'coordinates
            Call GridPress(AscW(Left(buffer, 1)) - 48, AscW(Right(buffer, 1)) - 48, entire)
            BufferExecute = False
        End If
    Else
        'shift has a value
        'check for another shift (turning the existing one off)
        If Left(buffer, 1) = "S" Then
            'oops, another shift
            If Asc(Right(buffer, 1)) - 48 = Shift Then
                Shift = 0 'user pressed twice: turn shift off
                'Debug.Print "Lock toggled off"
                BufferExecute = True
            Else
                'user has selected another shift
                Shift = Asc(Right(buffer, 1)) - 48
                'Debug.Print "Shift " & shift & " activated"
                BufferExecute = True
            End If
        Else
            If Shift = 1 Then
                'indicates a lock/unlock instruction
                Call ToggleLockNode(Asc(Left(buffer, 1)) - 47)
                Shift = 0
                BufferExecute = True
                'Debug.Print "Locked called"
            End If
        End If
    End If
    buffer = Empty
End Function

Private Sub ExecuteCommand(commandNumber As Integer)

    On Error Resume Next

    Dim i As Integer
    
    Debug.Print "CN: " & commandNumber
    If commandNumber = 1 Then
        'command for clear all locked nodes
        For i = LBound(cLockedNodes) To UBound(cLockedNodes)
            cLockedNodes(i) = False
        Next i
        Call CreateAxis
    End If
End Sub

Private Sub GridPress(x As Integer, y As Integer, entire As Collection)

    On Error Resume Next

'user has pressed button x,y in the grid: respond with information about connectivity
'nb x and y are 0-based: the axes are 1-based, since they are collections
    Dim connectionIterator As CConnection
    Dim targetID As String

    'Debug.Print "got x=" & x & " y=" & y
    'make sure presses are valid
    If x < 0 Or x > cAxis.Count - 1 Or y < 0 Or y > cAxis.Count - 1 Then
        'invalid coordinate: exit sub
        Exit Sub
    End If
    'work out to and from
    Set cToNode = cAxis(x + 1)
    Set cFromNode = cAxis(y + 1)
    'work out what to say
    cInformation = Empty
    'go through each connection looking for a connection from from to to
    For Each connectionIterator In cFromNode.connections
        'if this is a connectionEnd, check the parent node out
        If entire(connectionIterator.target).isConnectionEnd Then
            targetID = entire(connectionIterator.target).parent.id
        Else
            targetID = connectionIterator.target
        End If
        If targetID = cToNode.id Then
            'found a match!
            'now check direction: connectionIterator.direction has to match the node direction
            'okay, scrapped this for the time being
            'If connectionIterator.direction <> "from" Then
                cInformation = cInformation & connectionIterator.name & vbNewLine
            'End If
        End If
    Next connectionIterator
    'see if we found anything
    If cInformation = Empty Then
        'nope, put in a message to that effect
        cInformation = "No connection exists"
    End If
    'add names of from and to
    cInformation = "Connections between " & cFromNode.displayName & " and " & cToNode.displayName & vbNewLine & cInformation
End Sub

Public Property Get information() As String

    On Error Resume Next

'lets user get any information on the selected vertex
    information = cInformation
End Property

Private Sub Class_Initialize()

    On Error Resume Next

    cStartIndex = 1 ' by default, start at 1
    Shift = 0
End Sub
