VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CNode"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'Accessible UML

'LICENCE
'This program is free software: you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation, either version 3 of the License, or
'    (at your option) any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program.  If not, see <http://www.gnu.org/licenses/>.

'"Accessible UML" is copyright 2001-2004 Paul Blenkhorn and Gareth Evans, at UMIST,
'and is released by them under the GNU Public Licence Version 3. A copy is available
'at http://www.gnu.org/licenses/gpl.txt
'The code was written by Alasdair King.

Option Explicit

Private cChildren As Collection ' full child nodes
Private cName As String ' what we call this for the user
Private cNameSet As Boolean ' indicates whether the name has been set by the user (true) or should be generated (false)
Private cType As String
Private cID As String 'unique ID for this node
Private cParent As CNode
Private cAnnotation As String
Private cConnections As Collection      'this holds the IDs of all connected CNodes. For a connection-type node there will be only two
Private cConnectionEnds As Collection   'this holds the possible two connection ends
Private cAttributes As Collection
Private cNumber As Long ' the number of the item
Private cDirectionsID(17) As String  ' indicates the id of the item in X direction, if any
Private cDirectionsName(17) As String 'indicates the name of the item in X direction, if any
Private cCompassDirections(8) As String ' indicates the id of the item in X direction if any, numbering from 1 (north) clockwise to 8 (nw)
Private cPosition As CLocation ' location of the node
Private cIsAssociation As Boolean ' whether this node is an Assocation
Private cExits As Collection ' a composite of all the exits from the node
Private cPoints As Collection ' the points making up the node

Public Property Get isLastState() As Boolean
'tries to work out if this node is the final state in a statemachine and returns the answer
    On Error Resume Next
    Dim exitIterator As CExit
    Dim gotIn As Boolean
    
    If cType = UML_NODE_TYPE_FINALSTATE Then
        isLastState = True
    ElseIf cType = UML_NODE_TYPE_COMPOSITESTATE Or cType = UML_NODE_TYPE_SIMPLESTATE Then
        'have to have a connection if a final state
        If cExits.Count > 0 Then
            isLastState = True
        End If
        'check we don't link out
        For Each exitIterator In cExits
            If exitIterator.connectionType = "Transition target" Then
                isLastState = False
            End If
        Next exitIterator
    End If
End Property

Public Property Get isFirstState() As Boolean
'tries to work out if this node is the first state in a statemachine and returns the answer
    On Error Resume Next
    If cType = UML_NODE_TYPE_PSEUDOSTATE Then
        isFirstState = True
    End If
End Property

Public Property Get isDiagram() As Boolean
'returns whether this node represents the top node in a uml or floorplan diagram
    'DEV: This is NOT a very good way to do this because I don't manage the
    'possible cType values rationally.
    On Error Resume Next
    If cType = "Collaboration" Or cType = "Model" Or cType = UML_NODE_TYPE_STATEMACHINEDIAGRAM Then
        isDiagram = True
    End If
End Property

Public Property Get isGraphical() As Boolean
'returns whether the current node is able to be displayed graphically, e.g. in SVG
    On Error Resume Next
    If Right(cType, Len(cType) - InStrRev(cType, " ")) = "Diagram" Then
        isGraphical = True
    End If
End Property

Public Property Get exits() As Collection
    On Error Resume Next
    Set exits = cExits
End Property

Public Property Get isAssociation() As Boolean
    On Error Resume Next
    isAssociation = cIsAssociation
End Property

Public Property Get displayName() As String
    On Error Resume Next
    'a friendly name for the node to display
    displayName = cName & ": " & cType
End Property

Public Property Get nodeName() As String
    On Error Resume Next
    'returns the name of the node
    nodeName = cName
End Property

Public Property Let nodeName(newName As String)
    On Error Resume Next
    cName = newName
End Property

Public Property Get nodeID() As String
    On Error Resume Next
    nodeID = cID
End Property

Public Property Let nodeID(newID As String)
    On Error Resume Next
    If cID = Empty Then
        cID = newID
    End If
End Property


Public Property Get location() As CLocation
    On Error Resume Next
    Set location = cPosition
End Property

Public Property Get neighbourID(direction As Integer) As String
    On Error Resume Next
    neighbourID = cDirectionsID(direction)
End Property

Public Property Get neighbourName(direction As Integer) As String
    On Error Resume Next
    neighbourName = cDirectionsName(direction)
End Property

Public Sub CalculateCompassDirections(complete As Collection)
    On Error Resume Next
'this sub works out from the relationships array the contents of the cCompassDirections array. There is a
'strong possibility that the directions won't map correctly, because of the layout of the diagram. If so,
'a message is printed to debug, but no other action is taken
    Dim i As Integer
    Dim otherX As Integer
    Dim otherY As Integer
    Dim connectedNode As Object
    Dim calculatedDirection As Integer
    Dim otherID As String ' what we're going to assign
    'Debug.Print "Doing node id=" & cID
    'okay, go through the relationships and work out which direction each is in
    For i = 1 To cExits.Count
        'okay, so what is this item?
        Set connectedNode = complete(cExits(i).target)
        If connectedNode.isConnectionEnd Then
            If connectedNode.parent.id = cID Then
                'this is a connection to my own connectionEnd
                otherX = complete(connectedNode.target).x
                otherY = complete(connectedNode.target).y
            Else
                'this is a connection to another node's connectionEnd
                otherX = connectedNode.parent.location.x
                otherY = connectedNode.parent.location.y
            End If
        Else
            otherX = connectedNode.location.x
            otherY = connectedNode.location.y
        End If
        'Debug.Print "Connecting current node id=" & cID & " at (" & location.x & "," & location.y & ") to node id=" & connectedNode.id & " at (" & otherX & "," & otherY & ")"
        If cType = "Message sender" Or cType = "Message recipient" Then
            'special case (sigh) for sequence diagram messages
            If otherX = location.x Then
                'this is a message to itself: either it is the passage of
                'time, which is fine, or it is a send/receive
'                Debug.Print cExits(i).connectionType
                If cExits(i).connectionType <> "Later" And cExits(i).connectionType <> "Earlier" Then
                    'okay, it isn't a north/south time passes message: have to
                    'realign it
                    If cType = "Message sender" Then
                        'go to right
                        otherX = location.x + 1
                    ElseIf cType = "Message recipient" Then
                        'go to left
                        otherX = location.x - 1
                    End If
                End If
            Else
                'okay, message to left or right: have to make east/west
                otherY = location.y
            End If
        End If
        'ASSERTION: otherX and otherY are the coordinates of the other node to which this one is connected
        'we can use x and y, and nodeName, to get the location of the connectionEnd/its parent node, and the
        'name of the node
        'now work out the direction
        calculatedDirection = Int((CalculateAngleBetween(location.x, location.y, otherX, otherY) / 360# + 1 / 16) * 8)
        If calculatedDirection = 8 Then calculatedDirection = 0
        calculatedDirection = calculatedDirection + 1
        'Debug.Print "Got " & calculatedDirection
        'If calculatedDirection = 8 Then calculatedDirection = 7
        'calculatedDirection = calculatedDirection + 1
        'Debug.Print "Goes to " & calculatedDirection
        'Assuming that this has worked okay, what ID will we indicate the direction with?
        If connectedNode.isConnectionEnd Then
            If connectedNode.parent.id = cID Then
                'connected to my own connectionEnd: use the destination
                otherID = complete(connectedNode.target).id
            Else
                'connected to another node's connectionEnd
                otherID = connectedNode.id
            End If
        Else
            'connected straight to another node
            otherID = connectedNode.id
        End If
        'now test if we can make this assignment
        If cCompassDirections(calculatedDirection) <> Empty Then
            'oops, already assigned something to that compass direction: however, if it is the same thing,
            'we're okay (example: a relationship from a node, and another one two it: still creates one
            'compass direction, but assigns the same one twice)
            If cCompassDirections(calculatedDirection) = otherID Then
                'oh, okay, it's the same node: carry on
            Else
                'whoops, two competing nodes: at present, don't do anything except print a debugging error.
                'This is because building a document layout system is too difficult in the timescales available
                'Debug.Print "Failed to assign cleanly in CalculateCompassDirections"
                'Debug.Print Me.name & " to " & connectedNode.name
            End If
        End If
        'Make the assignment with the id
        cCompassDirections(calculatedDirection) = otherID
        'Debug.Print Me.name & " direction=" & calculatedDirection & " = " & otherID
    Next i
End Sub

Public Property Get compassID(direction As Integer) As String
    On Error Resume Next
'returns the id of a node in the given direction, if any
    compassID = cCompassDirections(direction)
End Property

Public Sub CalculateRelationshipDirections(complete As Collection)
    On Error Resume Next
'this sub works out from the relationships array the contents of the myDirections array.
    Dim i As Integer
    Dim otherX As Integer
    Dim otherY As Integer
    Dim connectedNode As Object
    Dim calculatedDirection As Integer
    
    'direction 16 is always oneself
    cDirectionsName(16) = cName
    cDirectionsID(16) = cID
'    If cName = "Mailbox" Then
 '       debugging = True
  '  End If
    'okay, go through the relationships and work out which direction each is in
'    Debug.Print "My type: " & cType
    For i = 1 To cExits.Count
    'For i = 1 To cConnections.Count
        'okay, so what is this item?
        Set connectedNode = complete(cExits(i).target)
        If connectedNode.isConnectionEnd Then
            If connectedNode.parent.id = cID Then
                'this is a connection to my own connectionEnd
                otherX = complete(connectedNode.target).x
                otherY = complete(connectedNode.target).y
            Else
                'this is a connection to another node's connectionEnd
                otherX = connectedNode.parent.location.x
                otherY = connectedNode.parent.location.y
            End If
        Else
            otherX = connectedNode.location.x
            otherY = connectedNode.location.y
        End If
        'Debug.Print "Me: (" & location.x & "," & location.y & ") Them: (" & otherX & "," & otherY & ")"
        'ASSERTION: otherX and otherY are the coordinates of the other node to which this one is connected
        'we can use x and y, and nodeName, to get the location of the connectionEnd/its parent node, and the
        'name of the node
        'now work out the direction
        If cType = "Message sender" Or cType = "Message recipient" Then
            'special case (sigh) for sequence diagram messages
            If otherX = location.x Then
                'this is a message to itself: either it is the passage of
                'time, which is fine, or it is a send/receive
'                Debug.Print cExits(i).connectionType
                If cExits(i).connectionType <> "Later" And cExits(i).connectionType <> "Earlier" Then
                    'okay, it isn't a north/south time passes message: have to
                    'realign it
                    If cType = "Message sender" Then
                        'go to right
                        otherX = location.x + 1
                    ElseIf cType = "Message recipient" Then
                        'go to left
                        otherX = location.x - 1
                    End If
                End If
            Else
                'okay, message to left or right: have to make east/west
                otherY = location.y
            End If
        End If
        'work out direction from 0 (north) to 15 (NW)
        calculatedDirection = (CDbl(CalculateAngleBetween(location.x, location.y, otherX, otherY)) / 360# * 16#) + 2
        If calculatedDirection > 15 Then calculatedDirection = calculatedDirection - 15
        'ASSERTION: calculatedDirection holds the direction of the other node
        'fill that item with the correct name and id
        If connectedNode.isConnectionEnd Then
            If connectedNode.parent.id = cID Then
                'connected to my own connectionEnd: use the destination
                cDirectionsName(calculatedDirection) = complete(connectedNode.target).displayName
                cDirectionsID(calculatedDirection) = connectedNode.target
            Else
                'connected to another node's connectionEnd
                cDirectionsName(calculatedDirection) = connectedNode.parent.displayName
                cDirectionsID(calculatedDirection) = connectedNode.id
            End If
        Else
            'connected straight to another node
            cDirectionsName(calculatedDirection) = connectedNode.displayName
            cDirectionsID(calculatedDirection) = connectedNode.id
        End If
    Next i
End Sub

Public Property Get number() As Long
    On Error Resume Next
    number = cNumber
End Property

Public Property Let number(newNumber As Long)
    On Error Resume Next
    cNumber = newNumber
End Property

Public Property Get connections() As Collection
    On Error Resume Next
    Set connections = cConnections
End Property

Public Sub DetermineName()
    On Error Resume Next
'names the node, if it hasn't one already
    Dim named As Boolean
    Dim connectionNode As CConnection
    If cNameSet Then
        'okay, the name has been set: leave it.
    Else
        'not set already
        If cType = UML_NODE_TYPE_PSEUDOSTATE Then
            'pseudostates in state chart diagrams have been observed to be start states when generated by
            'Poseidon UML. No other PsuedoState objects exist in these diagrams. So, assume that all
            'PseudoState nodes are start states.
            cName = "Start state"
        ElseIf cType = UML_NODE_TYPE_FINALSTATE Then
            'end state
            cName = "Final state"
        ElseIf cConnectionEnds.Count > 0 Then
            'association or connection node: name by the two destinations
            cName = "Association between " & complete.item(cConnectionEnds(1).target).name & " and " & complete.item(cConnectionEnds(2).target).name
        Else
            'no connection ends (e.g. Class Dependency) so name by connections if possible
            If cType = "Generalization" Then
                'this is a generalization, a specialised form of Association
                'get name of participants in generalization
                If cConnections.item(1).connectionType = "Generalization of" Then
                    cName = "Generalization of " & complete.item(cConnections.item(1).target).name & " by " & complete.item(cConnections.item(2).target).name
                Else
                    cName = "Generalization of " & complete.item(cConnections.item(2).target).name & " by " & complete.item(cConnections.item(1).target).name
                End If
            Else
                'don't handle this type specially: name from connections
                If cConnections.Count > 1 Then
                    cName = cType & " " & complete.item(cConnections(1).target).name & " and " & complete.item(cConnections(2).target).name
                Else
                    If cConnections.Count = 1 Then
                        cName = cType & " " & complete.item(cConnections(1).target).name
                    Else
                        ' no connections, no name: give it a type and number
                        cName = cType & " " & cID
                    End If
                End If
            End If
        End If
    End If
    'Debug.Print "Node named: " & cName
End Sub

Public Property Get connectionEnds() As Collection
    On Error Resume Next
    Set connectionEnds = cConnectionEnds
End Property

Public Property Get hasConnectionEnds() As Boolean
    On Error Resume Next
    If cConnectionEnds.Count > 0 Then
        hasConnectionEnds = True
    Else
        hasConnectionEnds = False
    End If
End Property

Public Property Get attributes() As Collection
    On Error Resume Next
    Set attributes = cAttributes
End Property


Public Property Get allNodes() As Collection
    On Error Resume Next
'return all the nodes in the diagram (strictly, all the nodes below and including this one)
    Dim all As New Collection
    Dim iterator As CNode
    Dim cIterator As CConnectionEnd
    Dim collIterator As Object
    
    Call all.add(Me, cID)
    For Each iterator In cChildren
        For Each collIterator In iterator.allNodes
            Call all.add(collIterator, collIterator.id)
        Next collIterator
    Next iterator
    For Each cIterator In cConnectionEnds
        Call all.add(cIterator, cIterator.id)
    Next cIterator
    Set allNodes = all
End Property

Public Property Get annotation() As String
    On Error Resume Next
    annotation = cAnnotation
End Property

Public Property Let annotation(newAnnotation As String)
    On Error Resume Next
    cAnnotation = newAnnotation
End Property

Public Property Get parent() As CNode
    On Error Resume Next
    Set parent = cParent
End Property

Public Property Set parent(newParent As CNode)
    On Error Resume Next
    Set cParent = newParent
End Property

Public Property Get childNodes() As Collection
    On Error Resume Next
    Set childNodes = cChildren
End Property

Public Property Set childNodes(newChildNodes As Collection)
    On Error Resume Next
    Set cChildren = newChildNodes
End Property

Public Property Get name() As String
    On Error Resume Next
    name = cName
End Property

Public Property Let name(newName As String)
    On Error Resume Next
    If Len(newName) > 0 Then
        cName = newName
        cNameSet = True
    End If
End Property

Public Property Get nodeType() As String
    On Error Resume Next
    nodeType = cType
End Property

Public Property Let nodeType(newType As String)
    On Error Resume Next
    cType = newType
End Property

Public Property Get id() As String
    On Error Resume Next
    id = cID
End Property

Public Property Let id(newID As String)
    On Error Resume Next
    cID = newID
End Property

Private Sub Class_Initialize()
    On Error Resume Next
    Set cChildren = New Collection
    Set cConnections = New Collection
    Set cAttributes = New Collection
    Set cConnectionEnds = New Collection
    Set cPosition = New CLocation
    Set cExits = New Collection
    Set cPoints = New Collection
End Sub

Public Property Get isConnectionEnd() As Boolean
    On Error Resume Next
    isConnectionEnd = False
End Property

Public Sub ParseXML(source As IXMLDOMNode)
'parses the XML node provided into the node tree
    On Error Resume Next
    Dim iterator As IXMLDOMNode
    Dim noteIterator As IXMLDOMNode
    Dim attributeNode As CAttribute
    Dim connectionEnd As CConnectionEnd
    Dim connectionNode As CConnection
    Dim newNode As CNode
    Dim checkName As String
    Dim i As Integer
    Dim newExit As CExit
    Dim newPoint As CPoint
    
    'do node attributes
    cName = source.attributes.getNamedItem("name").Text
    If Len(cName) > 0 Then cNameSet = True
    cType = source.attributes.getNamedItem("type").Text
    If cType = "Association" Then cIsAssociation = True
    cID = source.attributes.getNamedItem("id").Text
    'Debug.Print "Did cID=" & cID
    'do starting point
    If Not (source.attributes.getNamedItem("number") Is Nothing) Then
        If source.attributes.getNamedItem("number").Text <> Empty Then
            cNumber = CLng(source.attributes.getNamedItem("number").Text)
        End If
    End If
    For Each iterator In source.selectNodes("note")
        For Each noteIterator In iterator.childNodes
            If noteIterator.nodeType = NODE_TEXT Then
                cAnnotation = cAnnotation & noteIterator.nodeValue
            ElseIf noteIterator.nodeName = "br" Then
                cAnnotation = cAnnotation & vbNewLine
            End If
        Next noteIterator
        cAnnotation = cAnnotation & vbNewLine
    Next iterator
    'do node Attributes
    For Each iterator In source.selectNodes("attribute")
        Set attributeNode = New CAttribute
        Call attributeNode.ParseXML(iterator)
        Call cAttributes.add(attributeNode)
    Next iterator
    'do child nodes and assign their parent
    For Each iterator In source.selectNodes("node")
        'Debug.Print "parseXML child: " & iterator.attributes.getNamedItem("name").Text
        Set newNode = New CNode
        Call newNode.ParseXML(iterator)
        Set newNode.parent = Me
        'add the node to the children, in order of name: assume the stylesheet
        'did this for us already (pace code below)
        Call cChildren.add(newNode, newNode.id)
    Next iterator
    'do connections
    For Each iterator In source.selectNodes("connection")
        Set connectionNode = New CConnection
        connectionNode.target = iterator.attributes.getNamedItem("href").Text
        connectionNode.id = connectionNode.target & "-" & GetTickCount & Rnd
        'Debug.Print "Connection to " & connectionNode.target
        connectionNode.connectionType = iterator.attributes.getNamedItem("relationship").Text
        Call cConnections.add(connectionNode, connectionNode.id)
    Next iterator
    'do connectionEnds
    For Each iterator In source.selectNodes("connectionEnd")
        Set connectionEnd = New CConnectionEnd
        connectionEnd.target = iterator.attributes.getNamedItem("href").Text
        connectionEnd.id = iterator.attributes.getNamedItem("id").Text
        If Not (iterator.attributes.getNamedItem("multiplicity") Is Nothing) Then
            connectionEnd.multiplicity = iterator.attributes.getNamedItem("multiplicity").Text
        End If
        If Not (iterator.attributes.getNamedItem("direction") Is Nothing) Then
            connectionEnd.direction = iterator.attributes.getNamedItem("direction").Text
        End If
        connectionEnd.aggregation = iterator.attributes.getNamedItem("aggregation").Text
        connectionEnd.navigable = CBool(iterator.attributes.getNamedItem("navigable").Text)
        connectionEnd.endType = "Association"
        Set connectionEnd.parent = Me
        'now add to connectionEnds
        'Debug.Print "Created ce with target: " & connectionEnd.target
        Call cConnectionEnds.add(connectionEnd, connectionEnd.id)
        If cConnectionEnds.Count = 2 And cIsAssociation Then
            'add references from each to the other
            cConnectionEnds.item(1).other = cConnectionEnds.item(2).id
            cConnectionEnds.item(2).other = cConnectionEnds.item(1).id
        End If
    Next iterator
    Set connectionEnd = Nothing
    'now create the total list of exits from the combined connection and connectionEnd collections
    For Each connectionEnd In cConnectionEnds
        Set newExit = New CExit
        Set newExit.connectionEnd = connectionEnd
        Call cExits.add(newExit, newExit.id)
    Next connectionEnd
    For Each connectionNode In cConnections
        Set newExit = New CExit
        Set newExit.connection = connectionNode
        Call cExits.add(newExit, newExit.id)
    Next connectionNode
    'do locations on the map, if any
    If Not (source.selectSingleNode("location") Is Nothing) Then
        'node has a location on the map or locations
        For Each iterator In source.selectNodes("location")
'            If cName = "Mailbox" Then
'                Debug.Print "doing mailbox"
'                Debug.Print iterator.attributes.getNamedItem("x").Text
'                Debug.Print iterator.attributes.getNamedItem("y").Text
'            End If
            location.x = iterator.attributes.getNamedItem("x").Text
            location.y = iterator.attributes.getNamedItem("y").Text
        Next iterator
    End If
'    If location.placed Then
'        Debug.Print "Laid out " & cName & " at (" & location.x & "," & location.y & ")"
'    End If
    'add to diagramNodes: the id should be unique
    'Debug.Print "Adding " & cName & " id=[" & cID & "]"
    Call diagramNodes.add(Me, cID)
    Call complete.add(Me, cID)
    Exit Sub
End Sub

Public Sub AddChild(newChild As CNode)
    On Error Resume Next
    Call cChildren.add(newChild, newChild.id)
End Sub

Public Sub RemoveChild(oldChild As CNode)
'try to delete the given child
    Dim exitIterator As CExit
    Call cChildren.Remove(oldChild.id)
    'now take out all the connections to this child
    For Each exitIterator In oldChild.exits
        Call complete(exitIterator.target).RemoveConnectionsTo(oldChild)
    Next exitIterator
End Sub

Public Sub AddConnection(newConnection As CConnection)
    On Error Resume Next
    Dim newExit As New CExit
    
    Call cConnections.add(newConnection, newConnection.id)
    Set newExit.connection = newConnection
    Call cExits.add(newExit, newExit.id)
End Sub

Public Sub RemoveConnectionsTo(otherNode As CNode)
'go through all the connections removing everything that links to otherNode
    On Error Resume Next
    Dim connectionIterator As CConnection
    Dim connectionEndIterator As CConnectionEnd
    
    'check each of our connection ends
    For Each connectionEndIterator In cConnectionEnds
        If complete(connectionEndIterator.target).id = otherNode.id Then
            'found one that links to the target node
            Call RemoveConnection(connectionEndIterator)
        End If
    Next connectionEndIterator
    For Each connectionIterator In cConnections
        If connectionIterator.target = otherNode.id Then
            Call RemoveConnection(connectionIterator)
        End If
    Next connectionIterator
End Sub

Public Sub RemoveConnection(oldConnection As Object)
'try to remove the given connection
    If TypeOf oldConnection Is CConnection Then
        Call cConnections.Remove(oldConnection.id)
    ElseIf TypeOf oldConnection Is CConnectionEnd Then
        Call cConnectionEnds.Remove(oldConnection.id)
    End If
    'always remove from exits
    Call cExits.Remove(oldConnection.id)
    Debug.Print "Removed: " & oldConnection.name
End Sub

Public Sub LayoutSequenceDiagramNodes()
'this routine works out the x,y locations of each of its child nodes, since it is a
'sequence diagram Collaboration element. This is based on the name and time of the nodes
'This is because sequence diagrams are connected spatially differently from normal diagrams.
    On Error Resume Next
    Dim objectName As String
    Dim xCoord As Integer
    Dim yCoord As Integer
    Dim splitName() As String
    Dim objectLocations As New Dictionary
    Dim childIterator As CNode
    Dim previousChild As CNode
    
    'iterate through each child. If we find a node with a unique name, give it
    'a new X coord (iterating up in units from 1), store this name to X
    'relationship in objectLocations and give it a Y coord from the time
    xCoord = 0 ' start counting at 0
    For Each childIterator In cChildren
        objectName = childIterator.messageNodeName
        If objectLocations.Exists(objectName) Then
            'we've already encountered this: use the stored X value
        Else
            'nope, a new one: add to dictionary
            Call objectLocations.add(objectName, CStr(xCoord))
            xCoord = xCoord + 1
        End If
        'now use location to place: watch out, though, for messages to themselves.
        'The recipient node has to be moved along one unit if it the end of the same object's message to itself
        yCoord = -1
        If Not (previousChild Is Nothing) Then
            'we have a previous child: is it the source of this child?
            If previousChild.messageNodeName = childIterator.messageNodeName Then
                'it is the same object: has it the same time?
                If previousChild.nodeTime = childIterator.nodeTime Then
                    'yep. Move it down one so they are not at the same point
                    yCoord = childIterator.nodeTime + 1
                End If
            End If
        End If
        If yCoord = -1 Then yCoord = childIterator.nodeTime
        Call childIterator.location.Move(CInt(objectLocations.item(objectName)), yCoord)
        Set previousChild = childIterator
    Next childIterator
    'join up the nodes
    Call JoinUpSequenceDiagramNodes
End Sub

Private Sub JoinUpSequenceDiagramNodes()
    'this joins up the child nodes of a Collaboration node correctly.
    '(That is, a sequence diagram). Horizontal/east-west connections will be
    'handled by the fact that they already exist in the diagram. We have to
    'add vertical/north-south connections between different times
    'ASSUMPTION: we have already ordered the diagram nodes
    'ASSUMPTION: we have already called LayoutSequenceDiagramNodes
    Dim i As Integer
    Dim firstNode As CNode
    Dim secondNode As CNode
    Dim newConnection As CConnection
    
    'go through each node up to the second-last
    For i = 1 To cChildren.Count - 1
        'compare with the next one: if the same name, join up
        Set firstNode = cChildren.item(i)
'        Debug.Print "doing " & firstNode.name & " : " & firstNode.nodeType
        Set secondNode = cChildren.item(i + 1)
        If firstNode.messageNodeName = secondNode.messageNodeName Then
            'Debug.Print "Joining to later..."
            'yep, they should be joined up.
            Set newConnection = New CConnection
            newConnection.name = secondNode.messageNodeName
            newConnection.target = secondNode.id
            newConnection.id = secondNode.id & "-" & GetTickCount
            newConnection.connectionType = "Later"
            Call firstNode.AddConnection(newConnection)
            Set newConnection = New CConnection
            newConnection.name = firstNode.messageNodeName
            newConnection.target = firstNode.id
            newConnection.id = firstNode.id & "-" & GetTickCount
            newConnection.connectionType = "Earlier"
            Call secondNode.AddConnection(newConnection)
        End If
    Next i
End Sub

Public Property Get points() As Collection
    On Error Resume Next
    Set points = cPoints
End Property

Public Property Get distanceTo(otherNode As CNode) As Double
    On Error Resume Next
    distanceTo = Sqr((CLng(otherNode.location.x) - CLng(cPosition.x)) * (CLng(otherNode.location.x) - CLng(cPosition.x)) + (CLng(otherNode.location.y) - CLng(cPosition.y)) * (CLng(otherNode.location.y) - CLng(cPosition.y)))
End Property


Private Function GetLocation(point As String) As CLocation
    On Error Resume Next
    Dim newLocation As New CLocation
    Dim x As Integer
    Dim y As Integer
    
    x = Left(point, InStr(1, point, ","))
    y = Right(point, Len(point) - InStr(1, point, ","))
    Debug.Print "Converted " & point & " into " & location.x & "," & location.y
    Call newLocation.Move(x, y)
    Set GetLocation = newLocation
End Function

Public Property Get allowsChildren() As Boolean
    'indicates whether the current node allows child nodes
    On Error Resume Next
    If cType = "Package" Or cType = "Model" Then
        allowsChildren = True
    Else
        allowsChildren = False
    End If
End Property

Public Property Get allowsConnections() As Boolean
    On Error Resume Next
    If cType = "Class" Or cType = "Object" Or cType = "Package" Or cType = "Actor" Or cType = "UseCase" Then
        allowsConnections = True
    End If
End Property

Public Property Get allowedConnections() As Collection
    'returns the types of connection that this node is allowed to employ
    On Error Resume Next
    Dim con As New Collection
    
    Select Case nodeType
        Case "Class"
            Call con.add("Association")
            Call con.add("Generalization")
            Call con.add("Realization")
            Call con.add("Dependency")
            Call con.add("Abstraction")
        Case "Object"
            Call con.add("Link")
        Case "Package"
            Call con.add("Dependency")
        Case "Type"
            Call con.add("Generalization")
            Call con.add("Realization")
            Call con.add("Dependency")
        Case "Implementation class"
            Call con.add("Realization")
            Call con.add("Dependency")
            Call con.add("Generalization")
        Case "Actor"
            Call con.add("Communicate")
            Call con.add("Generalization")
            Call con.add("Dependency")
        Case "UseCase"
            Call con.add("Communicate")
            Call con.add("Extend")
            Call con.add("Include")
    End Select
    Set allowedConnections = con
End Property

Public Function CanMakeConnection(connectionType As String) As Boolean
    On Error Resume Next

    Dim i As Variant
    
    For Each i In allowedConnections
        If InStr(1, connectionType, i, vbBinaryCompare) = 1 Then
            CanMakeConnection = True
            Exit For
        End If
    Next i
End Function

Public Property Get allowsDeletion() As Boolean
    'explains whether this node is allowed to be deleted
    On Error Resume Next
    If cType <> "Model" Then
        allowsDeletion = True
    Else
        allowsDeletion = False
    End If
End Property

Public Sub RemoveConnectionEnd(connectionEnd As CConnectionEnd)
'try to remove the identified connectionEnd from this node
    On Error Resume Next
    Call cConnectionEnds.Remove(connectionEnd.id)
End Sub

Public Property Get nodeTime() As Long
'only applies to message origin/recipient nodes
    If cType = "Message sender" Or cType = "Message recipient" Then
        nodeTime = CLng(Right(cName, Len(cName) - InStrRev(cName, " ")))
        'Debug.Print "nodeTime=" & nodeTime
    Else
        nodeTime = 0
    End If
End Property

Public Property Get messageNodeName() As String
'works out the name of the node - the bit before the " at time " section of the name.
    On Error Resume Next
    messageNodeName = Left(cName, InStr(1, cName, " at time "))
    If Len(messageNodeName) > 0 Then
        messageNodeName = Left(messageNodeName, Len(messageNodeName) - 1)
    End If
End Property

Public Sub RenameAsDiagram()
'this node is a diagram, but not a type we recognise: give it a diagram type
    On Error Resume Next
    Dim i As Integer
    Dim newType As String
    Dim gotChar As String
    
    newType = Left(cType, 1)
    If Len(cType) > 1 Then
        For i = 2 To Len(cType)
            gotChar = Mid(cType, i, 1)
            If UCase(gotChar) = gotChar Then
                'upper case: add a space
                newType = newType & " " & gotChar
            Else
                newType = newType & gotChar
            End If
        Next i
    End If
    cType = newType & " Diagram"
End Sub
