<?xml version="1.0" encoding="UTF-8"?>
<!-- Converts version 1 TeDUB files from image analysis into the format I support -->
<xsl:stylesheet
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
      
      version="1.0">
	<xsl:output method="xml" omit-xml-declaration = "no"/>
	<xsl:template match="diagram">
       	<diagram>
       		<xsl:attribute name="type"><xsl:value-of select="type"/></xsl:attribute>
			<xsl:attribute name="languageVersion"><xsl:value-of select="@languageVersion"/></xsl:attribute>

       			<xsl:for-each select="contents">
       				<xsl:apply-templates/>	
       			</xsl:for-each>
		</diagram>
   	</xsl:template>
   	<xsl:template match="//node">
		<node>
			<!-- attributes of main node -->
			<xsl:attribute name="type"><xsl:value-of select="type"/></xsl:attribute>
			<xsl:attribute name="name"><xsl:value-of select="name"/></xsl:attribute>
			<xsl:if test="@isStartingPoint = 'y'">
				<xsl:attribute name="isStartingPoint">true</xsl:attribute>
			</xsl:if>
			<xsl:attribute name="id">
				<xsl:value-of select="cname"/>
				<xsl:if test="type = 'door' or type = 'window'">
					<!-- if a door or window, need to identify uniquely by linking ID to parent ID -->
					<xsl:text>-</xsl:text><xsl:value-of select="parent::*/parent::*/cname"/>
				</xsl:if>
			</xsl:attribute>
			<xsl:attribute name="number"><xsl:value-of select="@number"/></xsl:attribute>
			<xsl:for-each select="note"><note><xsl:value-of select="current()"/></note></xsl:for-each>
			<!-- annotation -->
			<xsl:for-each select="annotation">
				<note><xsl:value-of select="current()"/></note>
			</xsl:for-each>
			<!-- attribute nodes of main node -->
			<xsl:for-each select="data">
				<xsl:for-each select="item">
					<attribute>
						<xsl:attribute name="type">(Information)</xsl:attribute>
						<xsl:attribute name="name"><xsl:value-of select="current()"/></xsl:attribute>
						<xsl:attribute name="visibility"></xsl:attribute>
						<xsl:attribute name="value"></xsl:attribute>
					</attribute>
				</xsl:for-each>
			</xsl:for-each>
			<!-- connections: need to get each one, and correct the destination to be a true node, not a connection node -->
			<xsl:for-each select="connected">
				<!-- if a door, add a special connected node to the other node with the same cname -->
				<xsl:if test="parent::*/type = 'door'">
					<connection>
						<xsl:variable name="otherID"><xsl:value-of select="/diagram/contents/node/kids/node[kids/node/cname = current()/parent::*/cname and current()/parent::*/parent::*/parent::*/cname != cname]/cname"/></xsl:variable>
						<!-- there is a chance that the other id is blank, which occurs if this is a connection to outside. If so, add a link to oneself and set relationship to exit -->
						<xsl:choose>
							<xsl:when test="$otherID = ''">
								<!-- link outside! -->
								<xsl:attribute name="relationship">Exit</xsl:attribute>
								<xsl:attribute name="href">
									<xsl:value-of select="parent::*/cname"/><xsl:text>-</xsl:text><xsl:value-of select="parent::*/parent::*/parent::*/cname"/>
								</xsl:attribute>
							</xsl:when>
							<xsl:otherwise>
								<!-- okay, link to another door -->
								<xsl:attribute name="href">
									<xsl:value-of select="parent::*/cname"/>
									<xsl:text>-</xsl:text><xsl:value-of select="$otherID"/>
								</xsl:attribute>
								<xsl:attribute name="relationship">Travel</xsl:attribute>
							</xsl:otherwise>
						</xsl:choose>
					</connection>
				</xsl:if>
				<!-- Process other connections -->
				<xsl:for-each select="to">
					<xsl:variable name="myNode"><xsl:value-of select="current()/parent::*/parent::*/cname"/></xsl:variable>
					<xsl:variable name="myParent"><xsl:value-of select="current()/parent::*/parent::*/parent::*/parent::*/cname"/></xsl:variable>
					<!--<dummy><xsl:value-of select="$myParent"/></dummy>-->
					<xsl:variable name="myTarget"><xsl:value-of select="cname"/></xsl:variable>
					<!-- have to add extra differentiating code if door or window, and may not include connection -->
					<xsl:choose>
						<!-- DEV: nope, don't connect all windows  <xsl:when test="parent::*/parent::*/type = 'window'"> -->
						<!-- I am a connection from a window to something -->
						<!-- Always put these in: always use targetHref-parentCname as the target -->
<!--							<connection>
								<xsl:attribute name="href">
									<xsl:value-of select="cname"/><xsl:text>-</xsl:text><xsl:value-of select="parent::*/parent::*/parent::*/parent::*/cname"/>
								</xsl:attribute>
								<xsl:attribute name="relationship">Travel</xsl:attribute>
							</connection>
						</xsl:when> -->
						<xsl:when test="parent::*/parent::*/type = 'door' or parent::*/parent::*/type = 'window'">
							<!-- only include if connection to item in the same room: connections to items in other rooms are handled by the 
							     extra special connection added above -->
							<!-- test for other items in the same room -->
							<xsl:for-each select="/diagram/contents/node/kids/node/kids/node[cname = $myTarget]">
								<!--got a target of this href, check is same room -->
								<xsl:if test="parent::*/parent::*/cname = $myParent">
									<!-- item is in the same room -->
									<connection>
										<xsl:attribute name="href">
											<xsl:value-of select="cname"/><xsl:text>-</xsl:text><xsl:value-of select="$myParent"/>
										</xsl:attribute>
										<xsl:attribute name="relationship">Travel</xsl:attribute>
									</connection>
								</xsl:if>
							</xsl:for-each>
						</xsl:when>
						<xsl:otherwise>
							<!-- I am a connection neither from a window or a door (ie UML) - use unadulterated -->
							<connection>
								<xsl:attribute name="href"><xsl:value-of select="cname"/></xsl:attribute>
								<xsl:attribute name="relationship">Travel</xsl:attribute>
							</connection>							
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
			</xsl:for-each>
			<!-- get the shape of the room -->
			<xsl:copy-of select="polyline"/>
			<!-- get the location -->
			<xsl:copy-of select="location"/>
			<!-- Do child nodes -->
			<xsl:for-each select="kids">
   	  		<xsl:apply-templates/>	
   	  	</xsl:for-each>
	</node>
	</xsl:template>
</xsl:stylesheet>