<?xml version="1.0" encoding="UTF-8"?>
<!-- Converts version 2 image analysis files from Cornelius into my file format -->
<xsl:stylesheet
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
      
      version="1.0">
	<xsl:output method="xml" omit-xml-declaration = "no"/>
	<xsl:template match="diagram">
       	<diagram>
       		<xsl:attribute name="type"><xsl:value-of select="type"/></xsl:attribute>
			<xsl:attribute name="languageVersion"><xsl:value-of select="@languageVersion"/></xsl:attribute>
       			<xsl:for-each select="contents">
       				<xsl:apply-templates/>	
       			</xsl:for-each>
       		
		</diagram>
   	</xsl:template>
   	<xsl:template match="//node">
		<node>
			<!-- attributes of main node -->
			<xsl:attribute name="type"><xsl:value-of select="type"/></xsl:attribute>
			<xsl:attribute name="name"><xsl:value-of select="name"/></xsl:attribute>
			<xsl:if test="@isStartingPoint = 'y'">
				<xsl:attribute name="isStartingPoint">true</xsl:attribute>
			</xsl:if>
			<xsl:attribute name="id">
				<xsl:value-of select="cname"/>
				<xsl:if test="type = 'door' or type = 'window'">
					<!-- if a door or window, need to identify uniquely by linking ID to parent ID -->
					<xsl:text>-</xsl:text><xsl:value-of select="parent::*/parent::*/cname"/>
				</xsl:if>
			</xsl:attribute>
			<xsl:attribute name="number"><xsl:value-of select="@number"/></xsl:attribute>
			<xsl:for-each select="note"><note><xsl:value-of select="current()"/></note></xsl:for-each>
			<!-- annotation -->
			<xsl:for-each select="annotation">
				<note><xsl:value-of select="current()"/></note>
			</xsl:for-each>
			<!-- attribute nodes of main node -->
			<xsl:for-each select="data">
				<xsl:for-each select="item">
					<attribute>
						<xsl:attribute name="type">(Information)</xsl:attribute>
						<xsl:attribute name="name"><xsl:value-of select="current()"/></xsl:attribute>
						<xsl:attribute name="visibility"></xsl:attribute>
						<xsl:attribute name="value"></xsl:attribute>
					</attribute>
				</xsl:for-each>
			</xsl:for-each>
			<!-- connections -->
			<xsl:variable name="myID"><xsl:value-of select="cname"/></xsl:variable>
			<xsl:for-each select="//connection[@sourceNode = $myID]">
				<connection>
				<!-- <xsl:copy-of select="*"/> -->
					<xsl:choose>
						<xsl:when test="data/item[@what = 'Name']">
							<xsl:attribute name="relationship">
								<xsl:value-of select="current()/data/item[@what = 'Name']"/>
								<xsl:choose>
									<xsl:when test="@directed = 'y'">
										<xsl:text> to</xsl:text>
									</xsl:when>
									<xsl:otherwise>
										<xsl:text> with</xsl:text>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="relationship">
								<xsl:text>Undefined relationship</xsl:text>
								<xsl:choose>
									<xsl:when test="@directed = 'y'">
										<xsl:text> to</xsl:text>
									</xsl:when>
									<xsl:otherwise>
										<xsl:text> with</xsl:text>
									</xsl:otherwise>
								</xsl:choose>								
							</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:attribute name="href"><xsl:value-of select="@targetNode"/></xsl:attribute>
				</connection>
			</xsl:for-each>
			<xsl:for-each select="//connection[@targetNode = $myID]">
				<connection>
				<!-- <xsl:copy-of select="*"/> -->
					<xsl:choose>
						<xsl:when test="data/item[@what = 'Name']">
							<xsl:attribute name="relationship">
								<xsl:value-of select="current()/data/item[@what = 'Name']"/>
								<xsl:choose>
									<xsl:when test="@directed = 'y'">
										<xsl:text> from</xsl:text>
									</xsl:when>
									<xsl:otherwise>
										<xsl:text> with</xsl:text>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="relationship">
								<xsl:text>Undefined relationship</xsl:text>
								<xsl:choose>
									<xsl:when test="@directed = 'y'">
										<xsl:text> from</xsl:text>
									</xsl:when>
									<xsl:otherwise>
										<xsl:text> with</xsl:text>
									</xsl:otherwise>
								</xsl:choose>								
							</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:attribute name="href"><xsl:value-of select="@sourceNode"/></xsl:attribute>
				</connection>
			</xsl:for-each>
			<!-- get the shape of the room -->
			<xsl:copy-of select="polyline"/>
			<!-- get the location -->
			<xsl:copy-of select="location"/>
			<!-- Do child nodes -->
			<xsl:for-each select="kids">
   	  		<xsl:apply-templates/>	
   	  	</xsl:for-each>
	</node>
	</xsl:template>
	<xsl:template match="connection" priority="1"/>
</xsl:stylesheet>