<?xml version="1.0"?>
<xsl:stylesheet
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
      xmlns:UML="org.omg.xmi.namespace.UML"
      xmlns:svg="http://www.w3.org/2000/svg"
      version="1.0">
      <xsl:output method = "xml" omit-xml-declaration="no"/>
	<xsl:key name="nodeIndex" match="*[@xmi.id]" use="@xmi.id" />

      
         <xsl:template match="//XMI.header"></xsl:template>
	<xsl:template match="/">
       	<svg width="100%" height="100%" xmlns="http://www.w3.org/2000/svg">
   			<xsl:apply-templates/>
   		</svg>
   	</xsl:template>
   	
   	<xsl:template match="//XMI.content/UML:Diagram">
   		<g>
   			<xsl:attribute name="class">diagram</xsl:attribute>
   			<title><xsl:value-of select="@name"/></title>
   			<xsl:apply-templates select="UML:GraphElement.contained"/>
   		</g>
   	</xsl:template>
   	<xsl:template match="UML:GraphElement.contained/UML:GraphEdge">
   		<polyline>
   			<xsl:attribute name="stroke">
   				<xsl:text>grey</xsl:text>
   			</xsl:attribute>
			<xsl:attribute name="stroke-width">
				<xsl:text>2</xsl:text>
			</xsl:attribute>
			<xsl:attribute name="points">
				<xsl:for-each select="UML:GraphEdge.waypoints/XMI.field[XMI.field[1] != '0.0' or XMI.field[2] != '0.0']">
					<xsl:value-of select="XMI.field[1]"/>
			   		<xsl:text>, </xsl:text>
			   		<xsl:value-of select="XMI.field[2]"/>
					<xsl:text> </xsl:text>
   				</xsl:for-each>
   			</xsl:attribute>
   		</polyline>
   	</xsl:template>
   	<xsl:template match="UML:GraphElement.contained/UML:GraphNode[UML:GraphElement.semanticModel/UML:Uml1SemanticModelBridge]">
   		<xsl:for-each select="UML:GraphElement.semanticModel/UML:Uml1SemanticModelBridge/UML:Uml1SemanticModelBridge.element/*">
			<xsl:choose>
				<xsl:when test="local-name() = 'UseCase' or local-name() = 'Class' or local-name() = 'Actor' or local-name() = 'Object'">	
					<xsl:variable name="objectID"><xsl:value-of select="@xmi.idref"/></xsl:variable>
					<xsl:variable name="objectName"><xsl:value-of select="key('nodeIndex', $objectID)/@name"/></xsl:variable>
					<xsl:variable name="objectType"><xsl:value-of select="local-name()"/></xsl:variable>
					<xsl:for-each select="parent::*/parent::*/parent::*/parent::*">
						<xsl:variable name="umlX"><xsl:value-of select="UML:GraphElement.position/XMI.field[1]"/></xsl:variable>
						<xsl:variable name="umlY"><xsl:value-of select="UML:GraphElement.position/XMI.field[2]"/></xsl:variable>
						<xsl:variable name="umlWidth"><xsl:value-of select="UML:GraphNode.size/XMI.field[1]"/></xsl:variable>
						<xsl:variable name="umlHeight"><xsl:value-of select="UML:GraphNode.size/XMI.field[2]"/></xsl:variable>
						<xsl:variable name="umlXLeft"><xsl:value-of select="number($umlX - $umlWidth div 2)"/></xsl:variable>
						<xsl:variable name="umlYTop"><xsl:value-of select="number($umlY - $umlHeight div 2)"/></xsl:variable>
						<xsl:choose>
							<xsl:when test="$objectType = 'Class'">
								<rect>
						   			<xsl:attribute name="x">
						   			   	<xsl:value-of select="$umlX"/>
						   			</xsl:attribute>
						   			<xsl:attribute name="y">
						   			   	<xsl:value-of select="$umlY"/>
						   			</xsl:attribute>
						   			<xsl:attribute name="width">
						   			   	<xsl:value-of select="$umlWidth"/>
						   			</xsl:attribute>
						   			<xsl:attribute name="height">
						   			   	<xsl:value-of select="$umlHeight"/>
						   			</xsl:attribute>
						   			<xsl:attribute name="fill">
						   				<xsl:text>yellow</xsl:text>
						   			</xsl:attribute>
						   			<xsl:attribute name="stroke">
						   				<xsl:text>black</xsl:text>
						   			</xsl:attribute>
						   			<xsl:attribute name="stroke-width">
						   				<xsl:text>3</xsl:text>
						   			</xsl:attribute>
				   				</rect>
			   				</xsl:when>
			   				<xsl:when test="$objectType = 'UseCase'">
			   					<ellipse>
			   						<xsl:attribute name="cx">
						   			   	<xsl:value-of select="number($umlX + $umlWidth div 2)"/>
						   			</xsl:attribute>
						   			<xsl:attribute name="cy">
						   			   	<xsl:value-of select="number($umlY + $umlHeight div 2)"/>
						   			</xsl:attribute>
						   			<xsl:attribute name="rx">
						   			   	<xsl:value-of select="$umlWidth div 2"/>
						   			</xsl:attribute>
						   			<xsl:attribute name="ry">
						   			   	<xsl:value-of select="$umlHeight div 2"/>
						   			</xsl:attribute>
						   			<xsl:attribute name="fill">
						   				<xsl:text>green</xsl:text>
						   			</xsl:attribute>
						   			<xsl:attribute name="stroke">
						   				<xsl:text>black</xsl:text>
						   			</xsl:attribute>
						   			<xsl:attribute name="stroke-width">
						   				<xsl:text>3</xsl:text>
						   			</xsl:attribute>
			   					</ellipse>
			   				</xsl:when>
			   				<xsl:when test="$objectType = 'Actor'">
			   					<g>
			   						<xsl:attribute name="stroke">
							   			<xsl:text>grey</xsl:text>
							   		</xsl:attribute>
							   		<xsl:attribute name="stroke-width">
							   			<xsl:text>3</xsl:text>
							   		</xsl:attribute>
							   		<xsl:attribute name="fill">
							   			<xsl:text>none</xsl:text>
							   		</xsl:attribute>
			   						<polyline>
			   							<xsl:attribute name="points">
			   								<xsl:value-of select="$umlX"/>
			   								<xsl:text>, </xsl:text>
			   								<xsl:value-of select="$umlY + $umlHeight"/>
			   								<xsl:text> </xsl:text>

			   								<xsl:value-of select="number($umlX + $umlWidth div 2)"/>
			   								<xsl:text>, </xsl:text>
			   								<xsl:value-of select="number(2 * $umlHeight div 3 + $umlY)"/>
			   								<xsl:text> </xsl:text>
			   								
			   								<xsl:value-of select="number($umlX + $umlWidth)"/>
			   								<xsl:text>, </xsl:text>
			   								<xsl:value-of select="number($umlY + $umlHeight)"/>
			   								<xsl:text> </xsl:text>

			   								<xsl:value-of select="number($umlX + $umlWidth div 2)"/>
			   								<xsl:text>, </xsl:text>
			   								<xsl:value-of select="number(2 * $umlHeight div 3 + $umlY)"/>
			   								<xsl:text> </xsl:text>

			   								<xsl:value-of select="number($umlX + $umlWidth div 2)"/>
			   								<xsl:text>, </xsl:text>
			   								<xsl:value-of select="number($umlY + $umlHeight div 3)"/>
			   								<xsl:text> </xsl:text>

			   								<xsl:value-of select="number($umlX)"/>
			   								<xsl:text>, </xsl:text>
			   								<xsl:value-of select="number($umlY + $umlHeight div 3)"/>
			   								<xsl:text> </xsl:text>

			   								<xsl:value-of select="number($umlX + $umlWidth)"/>
			   								<xsl:text>, </xsl:text>
			   								<xsl:value-of select="number($umlY + $umlHeight div 3)"/>
			   								<xsl:text> </xsl:text>
			   											   													
			   							</xsl:attribute>
							   			
			   						</polyline>
			   						<circle>
			   							<xsl:attribute name="cx">
			   								<xsl:value-of select="number($umlX + $umlWidth div 2)"/>
			   							</xsl:attribute>
			   							<xsl:attribute name="cy">
			   								<xsl:value-of select="number($umlY + $umlHeight div 6)"/>
			   							</xsl:attribute>
			   							<xsl:attribute name="r">
			   								<xsl:value-of select="number($umlHeight div 6)"/>
			   							</xsl:attribute>
			   						</circle>
			   					</g>
			   				</xsl:when>
		   				</xsl:choose>
		   				<text>
		   					<xsl:attribute name="font-family">
		   						<xsl:text>Verdana</xsl:text>
		   					</xsl:attribute>
		   					<xsl:attribute name="font-weight">
		   						<xsl:text>normal</xsl:text>
		   					</xsl:attribute>
				   			<xsl:attribute name="x">
				   			   	<xsl:value-of select="number($umlX + $umlWidth div 2)"/>
				   			</xsl:attribute>
				   			<xsl:attribute name="y">
				   			   	<xsl:value-of select="number($umlY + $umlHeight div 2)"/>
				   			</xsl:attribute>
				   			<xsl:attribute name="text-anchor">
				   			   	<xsl:text>middle</xsl:text>
				   			</xsl:attribute>
				   			
				   			<!-- <xsl:attribute name="dx">
				   			   	<xsl:text>-</xsl:text><xsl:value-of select="number(string-length($objectName)) div 5"/><xsl:text>em</xsl:text>
				   			</xsl:attribute> -->
<!-- 				   			<xsl:attribute name="dy">
				   				<xsl:text>1em</xsl:text>
				   			</xsl:attribute> -->
							<xsl:value-of select="$objectName"/>
		   				</text>
	   				</xsl:for-each>
				</xsl:when>
			</xsl:choose>
		</xsl:for-each>
   	</xsl:template>
   	<xsl:template match="XMI.field" priority="1"/>
   	<xsl:template match="UML:Stereotype.baseClass"/>
   	<xsl:template match="UML:Model"/>
   	<xsl:template match="UML:GraphElement.contained/UML:GraphNode">
		<xsl:variable name="gcentreX"><xsl:value-of select="UML:GraphElement.position/XMI.field[1]"/></xsl:variable>
		<xsl:variable name="gcentreY"><xsl:value-of select="UML:GraphElement.position/XMI.field[2]"/></xsl:variable>
		<xsl:variable name="gwidth"><xsl:value-of select="UML:GraphNode.size/XMI.field[1]"/></xsl:variable>
		<xsl:variable name="gheight"><xsl:value-of select="UML:GraphNode.size/XMI.field[2]"/></xsl:variable>
		<xsl:variable name="gleft"><xsl:value-of select="number($gcentreX - $gwidth div 2)"/></xsl:variable>
		<xsl:variable name="gtop"><xsl:value-of select="number($gcentreY - $gheight div 2)"/></xsl:variable>
		<g>
			<xsl:attribute name="id">
				<xsl:value-of select="@xmi.id"/>
			</xsl:attribute>
 	   		<rect>
	   			<xsl:attribute name="x">
	   			   	<xsl:value-of select="$gleft"/>
	   			</xsl:attribute>
	   			<xsl:attribute name="y">
	   			   	<xsl:value-of select="$gtop"/>
	   			</xsl:attribute>
	   			<xsl:attribute name="width">
	   			   	<xsl:value-of select="$gwidth"/>
	   			</xsl:attribute>
	   			<xsl:attribute name="height">
	   			   	<xsl:value-of select="$gheight"/>
	   			</xsl:attribute>
	   			<xsl:attribute name="fill">
	   				<xsl:text>none</xsl:text>
	   			</xsl:attribute>
	   			<xsl:attribute name="stroke">
	   				<xsl:choose>
						<xsl:when test="UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement">none</xsl:when>
						<xsl:otherwise>black</xsl:otherwise>
					</xsl:choose>
	   			</xsl:attribute>
	   			<xsl:attribute name="stroke-width">
	   				<xsl:text>2</xsl:text>
	   			</xsl:attribute>
	   		</rect>
	   		<text>
				<xsl:attribute name="x">
	   			   	<xsl:value-of select="$gcentreX"/>
	   			</xsl:attribute>
	   			<xsl:attribute name="y">
	   			   	<xsl:value-of select="number($gcentreY+4)"/>
	   			</xsl:attribute>
	   			<!-- <xsl:attribute name="width">
	   			   	<xsl:value-of select="$gwidth"/>
	   			</xsl:attribute>
	   			<xsl:attribute name="height">
	   			   	<xsl:value-of select="$gheight"/>
	   			</xsl:attribute> -->
	   			<xsl:attribute name="text-anchor">
	   			   	<xsl:text>left</xsl:text>
				</xsl:attribute>
				<xsl:choose>
					<xsl:when test="UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement"/>
					<xsl:otherwise>
						<xsl:apply-templates select="UML:GraphElement.semanticModel"/>
					</xsl:otherwise>
				</xsl:choose>
			</text>
			<g>
				<xsl:attribute name="transform">
					<xsl:text>translate(</xsl:text>
					<xsl:value-of select="$gcentreX"/>
					<xsl:text>,</xsl:text>
					<xsl:value-of select="$gcentreY"/>
					<xsl:text>)</xsl:text>
				</xsl:attribute>
				<xsl:apply-templates select="UML:GraphElement.contained"/>
			</g>
		</g>
   	</xsl:template>
   	<xsl:template match="UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement">
   		<!-- <xsl:value-of select="@typeInfo"/> -->
   	</xsl:template>
   	<xsl:template match="UML:GraphElement.semanticModel/UML:Uml1SemanticModelBridge/UML:Uml1SemanticModelBridge.element">
   		<xsl:for-each select="*">
   			<!-- <xsl:value-of select="local-name()"/> -->
   			<xsl:value-of select="key('nodeIndex', @xmi.idref)/@name"/>
   		</xsl:for-each>
   	</xsl:template>   	
</xsl:stylesheet>
