<?xml version="1.0" encoding="UTF-8"?>
<!-- Conversion stylesheet for XMI 1.1 (e.g. Rational Rose) exported using Unisys.JCR.2 version 1.3.4 -->
<!-- STYLESHEET -->
<xsl:stylesheet
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
      xmlns:UML="href://org.omg/UML/1.3"
      version="1.0">
	<xsl:output method="xml" omit-xml-declaration = "yes" indent="yes" encoding="UTF-8"/>

	<!-- KEY INDICES -->
	<!-- Set-up an indexed table of the nodes in the file referenced by the unique xmi.id attribute -->
	<!-- Allows use of the efficient key() function -->
	<xsl:key name="nodeIndex" match="*[@xmi.id]" use="@xmi.id" />
	<xsl:key name="hrefIndex" match="*[@xmi.idref]" use="@xmi.idref" />
	<xsl:key name="clientIndex" match="//*[@client]" use="@client"/>
	<xsl:key name="supplierIndex" match="//*[@supplier]" use="@supplier"/>	
	<xsl:key name="annotationIndex" match="*[@annotatedElement]" use="@annotatedElement"/>
	<!-- MATCH XMI.header -->
	<!-- header element: contains file information we won't use -->
	<xsl:template match="/XMI/XMI.header"></xsl:template>
	
	<!-- CREATE diagram -->
	<!-- content section: this is the main content -->
	<xsl:template match="/XMI/XMI.content" priority="1">
		<diagram>
       		<xsl:attribute name="languageVersion">1.1</xsl:attribute>
       		<xsl:attribute name="type">uml</xsl:attribute>
       		<xsl:apply-templates select="UML:Model"/>
		</diagram>
   	</xsl:template>
  	<!-- CREATE node -->   	
    <xsl:template match="*[@xmi.id]">
    	<xsl:comment><xsl:value-of select="@name"/>:<xsl:value-of select="local-name()"/></xsl:comment>
		<node>
			<!-- CREATE node attributes -->
			<xsl:attribute name="type"><xsl:value-of select="local-name()"/></xsl:attribute>
			<xsl:attribute name="id"><xsl:value-of select="@xmi.id"/></xsl:attribute>
			<xsl:attribute name="number"><xsl:number/></xsl:attribute>
			<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
			<xsl:variable name="uniqueID" select="@xmi.id"/>
			<!-- CREATE note from Comment -->
			<xsl:for-each select="key('annotationIndex', $uniqueID)">
	    	   	<note>
   	   				<xsl:value-of select="@name"/>
	    	   	</note>
	    	</xsl:for-each>
			<!-- CREATE attribute from Attributes and Operations -->
			<xsl:for-each select="UML:Classifier.feature/*[local-name() != 'Method']">
				<attribute>
					<xsl:attribute name="type"><xsl:value-of select="local-name()"/></xsl:attribute>
					<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
					<xsl:attribute name="visibility"><xsl:value-of select="@visibility"/></xsl:attribute>
					
					<xsl:for-each select="UML:StructuralFeature.multiplicity">
						<xsl:call-template name="Multiplicity"/>
					</xsl:for-each>
					<xsl:attribute name="value">
						<xsl:variable name="targetID" select="@type"/>		
						<xsl:for-each select="key('nodeIndex', $targetID)">
							<xsl:value-of select="@name"/>
						</xsl:for-each>
					</xsl:attribute>
					<!-- CREATE parameter from Parameter (arguments to operations) -->
					<xsl:for-each select="UML:BehavioralFeature.parameter/UML:Parameter">
						<parameter>
							<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
							<xsl:attribute name="kind"><xsl:value-of select="@kind"/></xsl:attribute>
							<xsl:attribute name="type">
								<xsl:variable name="targetID" select="@type"/>			
								<xsl:for-each select="key('nodeIndex', $targetID)">
									<xsl:value-of select="@name"/>
								</xsl:for-each>
							</xsl:attribute>
						</parameter>
					</xsl:for-each>
				</attribute>
			</xsl:for-each>
			<!-- CREATE attribute from Stereotype -->
			<xsl:for-each select="key('nodeIndex', UML:ModelElement.stereotype/UML:Stereotype/@xmi.idref)">
			 	<attribute>
					<xsl:attribute name="type">Stereotype</xsl:attribute>
					<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
				</attribute>
			</xsl:for-each>
			<!-- Create connections for this node -->
			<xsl:choose>
				<!-- StateMachine: If this node is a StateMachine node, then this connections to/from this node should not be calculated, since it won't be represented -->
				<xsl:when test="local-name() = 'StateMachine'"/>
				<!-- Sequence diagram: -->
				<xsl:when test="local-name() = 'Collaboration'">
					<xsl:for-each select="UML:Namespace.ownedElement/UML:Object">
						<!-- 'got an object in the sequence diagram: now find all of the messages out of it. -->
						<xsl:variable name="objectID"><xsl:value-of select="@xmi.id"/></xsl:variable>
						<xsl:variable name="objectName"><xsl:value-of select="@name"/></xsl:variable>
						<xsl:for-each select="parent::*/UML:Stimulus[UML:Stimulus.sender/UML:Object/@xmi.idref = $objectID]">
							<xsl:variable name="stimulusID"><xsl:value-of select="@xmi.id"/></xsl:variable>
							<xsl:variable name="otherObjectID"><xsl:value-of select="UML:Stimulus.receiver/UML:Object/@xmi.idref"/></xsl:variable>
							<xsl:variable name="otherObjectName"><xsl:value-of select="parent::*/UML:Object[@xmi.id = $otherObjectID]/@name"/></xsl:variable>
							<xsl:variable name="actionID"><xsl:value-of select="UML:Stimulus.dispatchAction/*/@xmi.idref"/></xsl:variable>
							<xsl:comment><xsl:value-of select="@name"/>:Message sender</xsl:comment>
							<!-- CREATE node from sequence diagram -->
							<node>
								<xsl:attribute name="name">
									<!-- UML:Object name -->
									<xsl:for-each select="key('nodeIndex', $actionID)">
										<xsl:value-of select="$objectName"/>
									</xsl:for-each>
									<xsl:text> at time </xsl:text>
									<!-- time -->
								   	<xsl:for-each select="/XMI/XMI.content/UML:Diagram/UML:GraphElement.contained//UML:GraphNode[UML:GraphElement.semanticModel/UML:Uml1SemanticModelBridge/UML:Uml1SemanticModelBridge.element/*/@xmi.idref = $stimulusID]">
								   		<!-- //node has a single point location -->
								   		<xsl:for-each select="UML:GraphElement.position"><xsl:value-of select="substring-before(XMI.field[position() = 2], '.')"/></xsl:for-each>
								   	</xsl:for-each>
								</xsl:attribute>
								<xsl:attribute name="id">
									<xsl:value-of select="@xmi.id"/><xsl:text>:From</xsl:text>
								</xsl:attribute>
								<xsl:attribute name="number"><xsl:number/></xsl:attribute>
								<!-- //The name consists of the object name plus the time, given by the y-coordinate of the node!!!, plus any message from the CallAction node in XMI -->
								<xsl:attribute name="type">Message sender</xsl:attribute>
								<!-- CREATE connection -->
								<!-- now a connection to the other object -->
								<connection>
									<xsl:attribute name="href">
										<xsl:value-of select="@xmi.id"/><xsl:text>:To</xsl:text>
									</xsl:attribute>
									<xsl:attribute name="relationship">
										<xsl:value-of select="@name"/><xsl:text>: </xsl:text>
										<xsl:for-each select="UML:Stimulus.dispatchAction/*">
											<xsl:value-of select="local-name()"/>
										</xsl:for-each>
										<xsl:text> message sent to </xsl:text>
									</xsl:attribute>
								</connection>
							</node>
							<!-- Now create the node that the previous nodes links to - the other end of the message -->
					    	<xsl:comment><xsl:value-of select="@name"/>: Message recipient</xsl:comment>
					    	<!-- CREATE node -->
							<node>
								<xsl:attribute name="type">
									<xsl:text>Message recipient</xsl:text>
								</xsl:attribute>
								<xsl:attribute name="id">
									<xsl:value-of select="@xmi.id"/><xsl:text>:To</xsl:text>
								</xsl:attribute>
								<xsl:attribute name="number"><xsl:number/></xsl:attribute>
								<!-- The name consists of the object name plus the time, given by the y-coordinate of the node!!! -->
								<xsl:attribute name="name">
									<xsl:value-of select="$otherObjectName"/><xsl:text> at time </xsl:text>
									<xsl:for-each select="/XMI/XMI.content/UML:Diagram/UML:GraphElement.contained//UML:GraphNode[UML:GraphElement.semanticModel/UML:Uml1SemanticModelBridge/UML:Uml1SemanticModelBridge.element/*/@xmi.idref = $stimulusID]">
								   		<!-- node has a single point location -->
								   		<xsl:for-each select="UML:GraphElement.position"><xsl:value-of select="substring-before(XMI.field[position() = 2], '.')"/></xsl:for-each>
								   	</xsl:for-each>								
								</xsl:attribute>
								<!-- now a connection to the other object -->
								<connection>
									<xsl:attribute name="href">
										<xsl:value-of select="@xmi.id"/><xsl:text>:From</xsl:text>
									</xsl:attribute>
									<xsl:attribute name="relationship">
										<xsl:value-of select="@name"/><xsl:text>: </xsl:text>
										<xsl:for-each select="UML:Stimulus.dispatchAction/*">
											<xsl:value-of select="local-name()"/>
										</xsl:for-each>
										<xsl:text> message received from </xsl:text>
									</xsl:attribute>
								</connection>
							</node>
						</xsl:for-each>
					</xsl:for-each>
				</xsl:when>
				<!-- Connections for all other nodes -->
				<xsl:otherwise>
			    	<!-- 1 Connections originating at this node -->
			    	<!-- Associations: these are represented as connectionEnd nodes, since there is a great deal of information to store -->
				   	<xsl:for-each select="UML:Association.connection/UML:AssociationEnd">
				   		<connectionEnd>
				   			<xsl:attribute name="id"><xsl:value-of select="@xmi.id"/></xsl:attribute>
				   			<xsl:attribute name="href"><xsl:value-of select="attribute::type"/></xsl:attribute>
				   			<!-- These are values that can be of use to later calculations of relationship type -->
				   			<xsl:attribute name="aggregation"><xsl:value-of select="@aggregation"/></xsl:attribute>
				   			<xsl:attribute name="navigable"><xsl:value-of select="@isNavigable"/></xsl:attribute>
				   			<!-- Abandoned attempts to code compositions: leave it to the TeDUB system, easier than doing the XSL -->
				   			<!-- now the direction of the association. This is held in the values of the isNavigable attributes of the two connection ends -->
							<xsl:attribute name="direction">
								<xsl:choose>
									<xsl:when test="@isNavigable = 'false'">
										<!-- direction must be from -->
										<xsl:text>from</xsl:text>
									</xsl:when>
									<xsl:otherwise>
										<!-- direction could be to or both -->
										<xsl:choose>
											<xsl:when test ="parent::*/UML:AssociationEnd[@xmi.id != current()/@xmi.id]/@isNavigable = 'true'">
												<!-- other end is navigable, so both -->
												<xsl:text>both</xsl:text>
											</xsl:when>
											<xsl:otherwise>
												<!-- other end not navigable, so to -->
												<xsl:text>to</xsl:text>
											</xsl:otherwise>
										</xsl:choose>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:attribute>
							<!-- Multiplicity -->
							<xsl:for-each select="UML:AssociationEnd.multiplicity">
								<xsl:call-template name="Multiplicity"/>
							</xsl:for-each>
				   		</connectionEnd>
					</xsl:for-each>
				   	<!-- Client dependency -->
				   	<xsl:for-each select="@client">
				   		<connection>
							<xsl:attribute name="href"><xsl:value-of select="."/></xsl:attribute>
							<xsl:attribute name="relationship">Depends on</xsl:attribute>
							<xsl:attribute name="direction"><xsl:text>from</xsl:text></xsl:attribute>
						</connection>
				   	</xsl:for-each>
				   	<!-- Supplier dependency -->
				   	<xsl:for-each select="@supplier">
				   		<connection>
							<xsl:attribute name="href"><xsl:value-of select="."/></xsl:attribute>
							<xsl:attribute name="relationship">Depended on by</xsl:attribute>
							<xsl:attribute name="direction"><xsl:text> to </xsl:text></xsl:attribute>
						</connection>
				   	</xsl:for-each>
				   	<!-- Includes (Use cases) -->
				   	<xsl:for-each select="UML:Include.addition/*">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="attribute::xmi.idref"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Includes</xsl:attribute>
				   			<!-- <xsl:value-of select="//*[@xmi.id = current()/@xmi.idref]/@name"/> -->
				   		</connection>
				   	</xsl:for-each>
<!-- 				   	<xsl:for-each select="UML:UseCase.include/*">
				   		<connection><xsl:attribute name="href"><xsl:value-of select="attribute::xmi.idref"/></xsl:attribute>
				   		<xsl:attribute name="relationship">Includes</xsl:attribute>
				   		</connection>
				   	</xsl:for-each> -->
				   	<xsl:for-each select="UML:Include.base/*">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="attribute::xmi.idref"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Included by</xsl:attribute>
				   		</connection>
				   	</xsl:for-each>
					<!-- Extends (Use cases) -->
				   	<xsl:for-each select="@generalization">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="."/></xsl:attribute>
				   			<xsl:attribute name="relationship">Extension of</xsl:attribute>
				   		</connection>
				   	</xsl:for-each>
				   	<xsl:for-each select="@specialization">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="."/></xsl:attribute>
				   			<xsl:attribute name="relationship">Extended by</xsl:attribute>
				   		</connection>
				   	</xsl:for-each>
				   	<xsl:for-each select="@child">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="."/></xsl:attribute>
				   			<xsl:attribute name="relationship">Generalized by</xsl:attribute>
				   		</connection>
				   	</xsl:for-each>
				   	<xsl:for-each select="@parent">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="."/></xsl:attribute>
				   			<xsl:attribute name="relationship">Generalization of</xsl:attribute>
				   		</connection>
				   	</xsl:for-each>
				   	<!-- State machine context (State machines) -->
				   	<xsl:for-each select="UML:StateMachine.context/*">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="attribute::xmi.idref"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Context</xsl:attribute>
				   		</connection>
				   	</xsl:for-each>
				   	<!-- Transitions (State machines) -->
				   	<xsl:for-each select="UML:StateVertex.outgoing/*">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="attribute::xmi.idref"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Transition to </xsl:attribute>
				   		</connection>
				   	</xsl:for-each>
				   	<xsl:for-each select="UML:StateVertex.incoming/*">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="attribute::xmi.idref"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Transition from </xsl:attribute>
				   		</connection>
				   	</xsl:for-each>
				   	<xsl:for-each select="UML:Transition.source/*">
				   		<connection><xsl:attribute name="href"><xsl:value-of select="attribute::xmi.idref"/></xsl:attribute>
				   		<xsl:attribute name="relationship">Transition source</xsl:attribute>
				   		<!-- <xsl:value-of select="//*[@xmi.id = current()/@xmi.idref]/@name"/> -->
				   	</connection>
				   	</xsl:for-each>
				   	<xsl:for-each select="UML:Transition.target/*">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="attribute::xmi.idref"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Transition target</xsl:attribute>
				   			<!-- <xsl:value-of select="//*[@xmi.id = current()/@xmi.idref]/@name"/> -->
				   		</connection>
				   	</xsl:for-each>
<!--				   	<xsl:for-each select="UML:Message.conformingStimulus/*">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="attribute::xmi.idref"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Message</xsl:attribute>
				   		</connection>
				   	</xsl:for-each>
				   	<xsl:for-each select="UML:Stimulus.sender/*">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="attribute::xmi.idref"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Message sender</xsl:attribute>
				   		</connection>
				   	</xsl:for-each>
				   	<xsl:for-each select="UML:Stimulus.receiver/*">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="attribute::xmi.idref"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Message receiver</xsl:attribute>
				   		</connection>
				   	</xsl:for-each> -->
					<!-- <xsl:for-each select="UML:Generalization.child/*">
						<connection>
							<xsl:attribute name="href"><xsl:value-of select="@xmi.idref"/></xsl:attribute>
							<xsl:attribute name="relationship">Generalized by</xsl:attribute> 
							<xsl:attribute name="direction"><xsl:text>by</xsl:text></xsl:attribute>
						</connection>
					</xsl:for-each>
					<xsl:for-each select="UML:Generalization.parent/*">
						<connection>
							<xsl:attribute name="href"><xsl:value-of select="@xmi.idref"/></xsl:attribute>
							<xsl:attribute name="relationship">Generalization of</xsl:attribute>
							<xsl:attribute name="direction"><xsl:text>of</xsl:text></xsl:attribute>
		
						</connection>
				   	</xsl:for-each> -->
				   	<!-- 2 Connection information originating at another node, when another node is a connection node -->
				   	<!-- Associations linking to me -->
				   	<xsl:variable name="myType"><xsl:value-of select="local-name()"/></xsl:variable>
				   	<xsl:for-each select="parent::*/UML:Association/UML:Association.connection/UML:AssociationEnd[@type = $uniqueID]">
				   		<!-- this also gets use case diagrams usecase-actor relationships, which should be expressed differently -->
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="@xmi.id"/></xsl:attribute>
				   			<xsl:attribute name="relationship">
				   				<xsl:choose>
				   					<xsl:when test="local-name() = 'Actor' and $myType = 'UseCase'">
				   						<xsl:text>Uses</xsl:text>
				   					</xsl:when>
				   					<xsl:when test="local-name() = 'UseCase' and $myType = 'Actor'">
				   						<xsl:text>Used by</xsl:text>
				   					</xsl:when>
				   					<xsl:otherwise>
				   						<xsl:text>Association</xsl:text>
				   					</xsl:otherwise>
				   				</xsl:choose>
				   			</xsl:attribute>
				   		</connection>
				   	</xsl:for-each>
				   	<!-- Dependency, Extend and Include nodes linking to me -->
				   	<xsl:for-each select="key('clientIndex', $uniqueID)">
				   		<connection>
							<xsl:attribute name="href"><xsl:value-of select="@xmi.id"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Dependency client</xsl:attribute>
							<xsl:attribute name="direction"><xsl:text>from</xsl:text></xsl:attribute>
				   		</connection>
			   		</xsl:for-each>
			   		<xsl:for-each select="key('supplierIndex', $uniqueID)">
				   		<connection>
							<xsl:attribute name="href"><xsl:value-of select="@xmi.id"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Dependency supplier</xsl:attribute>
							<xsl:attribute name="direction"><xsl:text>to</xsl:text></xsl:attribute>
				   		</connection>			   			
			   		</xsl:for-each>
			   		<!-- DEV: Need to do extend and include for Rose XMI 1.1
				   	<xsl:for-each select="key('hrefIndex', $uniqueID)">
				   		<xsl:variable name="parentName"><xsl:for-each select="parent::*"><xsl:value-of select="local-name()"/></xsl:for-each></xsl:variable>
				   		<xsl:variable name="grandparentName"><xsl:for-each select="parent::*/parent::*"><xsl:value-of select="local-name()"/></xsl:for-each></xsl:variable>
				   		<xsl:choose>
				   			<xsl:when test="$parentName = 'Dependency.client'">
				   			</xsl:when>
				   			<xsl:when test="$parentName = 'Dependency.supplier'">
				   			</xsl:when>
				   			<xsl:when test="$grandparentName = 'Extend'">
						   		<connection>
									<xsl:attribute name="href"><xsl:value-of select="parent::*/parent::*/@xmi.id"/></xsl:attribute>
						   			<xsl:attribute name="relationship">Extends</xsl:attribute>
						   		</connection>
				   			</xsl:when>
				   			<xsl:when test="$grandparentName = 'Include'">
						   		<connection>
									<xsl:attribute name="href"><xsl:value-of select="parent::*/parent::*/@xmi.id"/></xsl:attribute>
						   			<xsl:attribute name="relationship">Includes</xsl:attribute>
						   		</connection>
				   			</xsl:when>
				   		</xsl:choose>
				   	</xsl:for-each>
				   	-->
					<!-- Messages (Sequence diagrams) -->
<!-- 					<xsl:for-each select="ancestor::UML:Namespace.ownedElement//UML:Stimulus[child::*/child::*/@xmi.idref = $uniqueID]">
				   		<connection>
				   			<xsl:attribute name="href"><xsl:value-of select="@xmi.id"/></xsl:attribute>
				   			<xsl:attribute name="relationship">Message</xsl:attribute>
				   		</connection>
					</xsl:for-each> -->
				    <!-- Match Generalization objects -->
				    <xsl:for-each select="ancestor::UML:Namespace.ownedElement//UML:Generalization[child::*/child::*/@xmi.idref = $uniqueID]">
				    	<!-- found a Generalization that links to me -->
				    	<xsl:for-each select="child::*">
				    		<xsl:if test="*/@xmi.idref = $uniqueID">
				    			<!-- found the end pointing at this node:get the other end -->
				    			<xsl:variable name="otherEnd"><xsl:value-of select="parent::node()/*/*[(@xmi.idref != $uniqueID) and (local-name() != 'Stereotype')]/@xmi.idref"/></xsl:variable>
				    			<xsl:if test="local-name() = 'Generalization.child'">
				    				<!-- check whether it is a class diagram ("inherit") or use case diagram ("extend") -->
				    				<xsl:if test="UML:Class">
					    			 	<connection><xsl:attribute name="href"><xsl:value-of select="$otherEnd"/></xsl:attribute><xsl:attribute name="relationship">Inherits from</xsl:attribute></connection>
					    			</xsl:if>
					    			<xsl:if test="UML:UseCase">
					    				<connection><xsl:attribute name="href"><xsl:value-of select="$otherEnd"/></xsl:attribute><xsl:attribute name="relationship">Extends</xsl:attribute></connection>
					    			</xsl:if>
					    		</xsl:if>
					    		<xsl:if test="local-name() = 'Generalization.parent'">
					    			<xsl:if test="UML:Class">
					    			 	<connection><xsl:attribute name="href"><xsl:value-of select="$otherEnd"/></xsl:attribute><xsl:attribute name="relationship">Superclass of</xsl:attribute></connection>
					    			</xsl:if>
					    			<xsl:if test="UML:UseCase">
					    				<connection><xsl:attribute name="href"><xsl:value-of select="$otherEnd"/></xsl:attribute><xsl:attribute name="relationship">Extended by</xsl:attribute></connection>
					    			</xsl:if>
					    		</xsl:if>
				    		</xsl:if>
				    	</xsl:for-each>
				    </xsl:for-each>
					
					<!-- 3 Location on the diagram, from Rose output -->
				   	<xsl:for-each select="/XMI/XMI.content/UML:Diagram/UML:Diagram.element/UML:DiagramElement[@subject = $uniqueID]">
				   		<!-- node has a single point location -->
				   		<xsl:variable name="geometry1">
				   			<xsl:value-of select="@geometry"/>
				   		</xsl:variable>
				   		<xsl:variable name="left">
				   			<xsl:value-of select="substring-before($geometry1, ',')"/>
				   		</xsl:variable>
				   		<xsl:variable name="geometry2">
				   			<xsl:value-of select="substring-after($geometry1, ',')"/>
				   		</xsl:variable>
				   		<xsl:variable name="top">
				   			<xsl:value-of select="substring-before($geometry2, ',')"/>				   			
				   		</xsl:variable>
				   		<xsl:variable name="geometry3">
				   			<xsl:value-of select="substring-after($geometry2, ',')"/>
				   		</xsl:variable>
				   		<xsl:variable name="width">
				   			<xsl:value-of select="substring-before($geometry3, ',')"/>
				   		</xsl:variable>
				   		<xsl:variable name="geometry4">
				   			<xsl:value-of select="substring-after($geometry3, ',')"/>
				   		</xsl:variable>
				   		<xsl:variable name="height">
				   			<xsl:value-of select="substring-before($geometry4, ',')"/>
				   		</xsl:variable>
				   		<l><xsl:value-of select="$left"/></l>
				   		<t><xsl:value-of select="$top"/></t>
				   		<w><xsl:value-of select="$width"/></w>
				   		<h><xsl:value-of select="$height"/></h>
				   		<location>
				   			<xsl:attribute name="x"><xsl:value-of select="number($left + $width div 2)"/></xsl:attribute>
			   				<xsl:attribute name="y"><xsl:value-of select="number($top + $height div 2)"/></xsl:attribute>
			   			</location>
				   	</xsl:for-each>
				   	<xsl:for-each select="/XMI/XMI.content/UML:Diagram/UML:GraphElement.contained//UML:GraphEdge[UML:GraphElement.semanticModel/UML:Uml1SemanticModelBridge/UML:Uml1SemanticModelBridge.element/*/@xmi.idref = $uniqueID]">
				   		<!-- node consists of a path -->
				   		<xsl:for-each select="UML:GraphEdge.waypoints/XMI.field">
				   			<xsl:if test="not(starts-with(XMI.field[position() = 1], '0')) or not(starts-with(XMI.field[position() = 2], '0'))">
						   		<location>
						   			<xsl:attribute name="x"><xsl:value-of select="XMI.field[position() = 1]"/></xsl:attribute>
					   				<xsl:attribute name="y"><xsl:value-of select="XMI.field[position() = 2]"/></xsl:attribute>
					   			</location>
				   			</xsl:if>
				   		</xsl:for-each>
				   	</xsl:for-each>
		   		</xsl:otherwise>
		   	</xsl:choose>
		   	<!-- if a Class, look for any attached State diagrams, and insert them here -->
		   	<xsl:for-each select="/XMI/XMI.content/UML:StateMachine[UML:StateMachine.context/*/@xmi.idref = $uniqueID]">
		   		<!-- found a state machine node matching:create the whole node, then do the kids -->
   		    	<xsl:comment><xsl:value-of select="@name"/>:State machine</xsl:comment>
		   		<node>
   					<!-- basic node information -->
		   			<xsl:attribute name="id"><xsl:value-of select="@xmi.id"/></xsl:attribute>
					<xsl:attribute name="number"><xsl:number/></xsl:attribute>
					<xsl:attribute name="name">State Machine diagram</xsl:attribute>
					<xsl:attribute name="type">State Machine</xsl:attribute>
					<!-- now do the state machine contents -->
					<xsl:for-each select="UML:StateMachine.top/UML:CompositeState/UML:CompositeState.subvertex">
						<xsl:apply-templates select="*"><xsl:sort select="node/@name"/></xsl:apply-templates>
					</xsl:for-each>
					<xsl:for-each select="UML:StateMachine.transitions">
						<xsl:apply-templates select="*"><xsl:sort select="node/@name"/></xsl:apply-templates>
					</xsl:for-each>
		   		</node>
		   	</xsl:for-each>
		   	<!-- Get the child nodes, if any -->
		   	<!-- <xsl:apply-templates select="parent::*/UML:Diagram[UML:Diagram.owner/UML:Uml1SemanticModelBridge/UML:Uml1SemanticModelBridge.element/UML:Model/@xmi.idref = $uniqueID]"/>-->
		   	<xsl:apply-templates select="parent::*/UML:Diagram"/>
		   	<!-- - all diagrams -->
	    	<xsl:apply-templates select="UML:Namespace.ownedElement/*"><xsl:sort select="@name"/></xsl:apply-templates>
	    	<!-- - state chart diagrams -->
	    	<xsl:apply-templates select="UML:CompositeState.subvertex/*"><xsl:sort select="@name"/></xsl:apply-templates>
    	</node>
    </xsl:template>
    <!-- CREATE Multiplicity results -->
    <xsl:template name="Multiplicity"><!-- match="UML:StructuralFeature.multiplicity/UML:Multiplicity"> -->
    	<xsl:for-each select="UML:Multiplicity">
    		<xsl:attribute name="multiplicity">
				<xsl:variable name="lower" select="UML:Multiplicity.range/UML:MultiplicityRange/@lower"/>
				<xsl:variable name="upper" select="UML:Multiplicity.range/UML:MultiplicityRange/@upper"/>
				<xsl:choose>
					<xsl:when test="$upper = '-1'"><xsl:value-of select="$lower"/><xsl:text> to n</xsl:text></xsl:when>
					<xsl:otherwise>
						<xsl:choose>
							<xsl:when test="$upper = $lower"><xsl:value-of select="$lower"/></xsl:when>
							<xsl:otherwise><xsl:value-of select="$lower"/><xsl:text> to </xsl:text><xsl:value-of select="$upper"/></xsl:otherwise>
						</xsl:choose>
					</xsl:otherwise>
				</xsl:choose>
	    	</xsl:attribute>
    	</xsl:for-each>
    </xsl:template>
    <!-- CREATE layout information from Diagram -->
    <xsl:template match="UML:Diagram" priority="1">
      	<xsl:comment><xsl:value-of select="@name"/>: Diagram</xsl:comment>
      	<!-- Create node representing diagram -->
    	<node>
    		<xsl:variable name="diagramType"><xsl:value-of select="@diagramType"/></xsl:variable>
			<xsl:attribute name="type">
				<xsl:choose>
					<xsl:when test="$diagramType = 'ClassDiagram'">
						<xsl:text>Class Diagram</xsl:text>
					</xsl:when>
					<xsl:when test="$diagramType = 'UseCaseDiagram'">
						<xsl:text>Use Case Diagram</xsl:text>
					</xsl:when>
					<xsl:when test="$diagramType = 'SequenceDiagram'">
						<xsl:text>Sequence Diagram</xsl:text>
					</xsl:when>
					<xsl:when test="$diagramType = 'StateDiagram'">
						<xsl:text>State Diagram</xsl:text>
					</xsl:when>
					<xsl:when test="$diagramType = 'ActivityDiagram'">
						<xsl:text>Activity Diagram</xsl:text>
					</xsl:when>
					<xsl:when test="$diagramType = 'CollaborationDiagram'">
						<xsl:text>Collaboration Diagram</xsl:text>
					</xsl:when>
					<xsl:when test="$diagramType = 'DeploymentDiagram'">
						<xsl:text>Deployment Diagram</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>Diagram</xsl:text>
					</xsl:otherwise>
				</xsl:choose>	
			</xsl:attribute>
			<xsl:attribute name="id"><xsl:value-of select="@xmi.id"/></xsl:attribute>
			<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
			<xsl:for-each select="descendant::UML:TextElement">
				<xsl:element name="attribute">
					<xsl:attribute name="type">Diagram text</xsl:attribute>
					<xsl:attribute name="name">
						<xsl:value-of select="@text"/>
					</xsl:attribute>
				</xsl:element>
			</xsl:for-each>
			<!-- link to contents of diagram: handled differently for sequence diagrams -->
			<xsl:choose>
				<!-- DEV: Need to sort sequence diagram children for Rose XMI 1.1. -->
				<xsl:when test="@diagramType = 'SequenceDiagram'">
					<xsl:variable name="collaborationID"><xsl:value-of select="@owner"/></xsl:variable>
					<xsl:for-each select="key('nodeIndex', $collaborationID)">
						<!--<xsl:for-each select="UML:CollaborationInstanceSet.participatingInstance/*"> -->
						<xsl:for-each select="parent::*/UML:Object">
					      	<xsl:comment><xsl:value-of select="@name"/>: Sequence diagram member</xsl:comment>	
						 	<node>
						 		<xsl:variable name="instanceType"><xsl:value-of select="local-name()"/></xsl:variable>
						 		<!-- <xsl:for-each select="key('nodeIndex', @xmi.idref)"> -->
									<xsl:variable name="objectTypeID"><xsl:value-of select="UML:Instance.classifier/*/@xmi.idref"/></xsl:variable>
									<xsl:attribute name="type"><xsl:value-of select="$instanceType"/></xsl:attribute>
									<xsl:attribute name="id"><xsl:value-of select="@xmi.id"/><xsl:text>-diagramcontent</xsl:text></xsl:attribute>
									<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
								<!-- </xsl:for-each> -->
								<!-- connections to messages - leave to importing program... -->
								<!-- connection to actual class, if any - put in here -->
								<xsl:for-each select="key('nodeIndex', $objectTypeID)">
									<connection>
										<xsl:attribute name="href">
												<xsl:value-of select="$objectTypeID"/>	
										</xsl:attribute>
							   			<xsl:attribute name="relationship">Instance of</xsl:attribute>									
									</connection>
								</xsl:for-each>
						 	</node>
						</xsl:for-each>
					</xsl:for-each> 
				</xsl:when>
				<!-- Non-sequence diagrams -->
				<xsl:otherwise>
					<xsl:for-each select="UML:Diagram.element/UML:DiagramElement">
						<xsl:variable name="layoutTarget"><xsl:value-of select="@subject"/></xsl:variable>
						<xsl:for-each select="key('nodeIndex', $layoutTarget)">
							<xsl:if test="starts-with(@xmi.id, 'S')">
					   			<connection>
					   				<xsl:attribute name="href"><xsl:value-of select="@xmi.id"/></xsl:attribute>
						   			<xsl:attribute name="relationship"><xsl:text>Contains</xsl:text></xsl:attribute>
						   		</connection>
					   		</xsl:if>
						</xsl:for-each>
				   	</xsl:for-each>			 
				</xsl:otherwise>
		   	</xsl:choose>
		</node>
    </xsl:template>
    <!-- Match/hide irrelevant elements -->
    <xsl:template match="*[@xmi.id]" priority="-2"/>
    <!-- Match/hide some types -->
    <xsl:template match="UML:DataType" priority="1"/>
    <xsl:template match="UML:Comment" priority="1"/>
    <xsl:template match="UML:Stereotype" priority="1"/>
 </xsl:stylesheet>