<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:UML="org.omg.xmi.namespace.UML" version="1.0">
	<xsl:output method="xml" omit-xml-declaration="yes" indent="yes" encoding="UTF-8"/>
	<xsl:key name="nodeIndex" match="*[@xmi.id]" use="@xmi.id"/>

	<!-- TEMPLATE: root XMI node of file -->
	<xsl:template match="XMI">
		<diagram>
       		<xsl:attribute name="languageVersion">1.2</xsl:attribute>
       		<xsl:attribute name="type">uml</xsl:attribute>
       		<xsl:apply-templates/>
		</diagram>
	</xsl:template>
	
	<!-- TEMPLATE: UML Models in the root XMI node -->
	<xsl:template match="UML:Model">
		<node>
			<xsl:attribute name="type">Diagram</xsl:attribute>
			<xsl:attribute name="id"><xsl:value-of select="@xmi.id"/></xsl:attribute>
			<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
			<xsl:apply-templates/>
		</node>		
	</xsl:template>
	
	<!-- UML diagrams -->
    <xsl:template match="UML:Diagram" priority="1">
    	<node>
			<xsl:attribute name="type">Diagram</xsl:attribute>
			<xsl:attribute name="id"><xsl:value-of select="@xmi.id"/></xsl:attribute>
			<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
			<note><xsl:text>Diagram contains the following items: </xsl:text>
			<xsl:for-each select="UML:GraphElement.contained/UML:GraphNode/UML:GraphElement.semanticModel/UML:Uml1SemanticModelBridge/UML:Uml1SemanticModelBridge.element/*">
				<xsl:variable name="target" select="@xmi.idref"/>			
				<xsl:for-each select="key('nodeIndex', $target)">
					<xsl:value-of select="@name"/>
				</xsl:for-each>
				<xsl:text>. </xsl:text>
			</xsl:for-each>
			</note>
		</node>
    </xsl:template>	
    
    <!-- Main file header -->
    <xsl:template match="XMI.header"></xsl:template>
    	
    <!-- Main file content -->
	<xsl:template match="XMI.content">
       	<xsl:apply-templates select="UML:Model"/>
   	</xsl:template>
   	
   	<!-- UML Nodes -->
   	<xsl:template match="*[@xmi.id]">
    	<node>
			<xsl:attribute name="type"><xsl:value-of select="local-name()"/></xsl:attribute>
			<xsl:attribute name="id"><xsl:value-of select="@xmi.id"/></xsl:attribute>
			<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>    		
       		<xsl:apply-templates />
    	</node>
    </xsl:template>
    
    <!-- Tagged value? -->
    <xsl:template match="UML:TaggedValue.dataValue">
    	<note><xsl:value-of select="."/></note>
    </xsl:template>
    
</xsl:stylesheet>