Attribute VB_Name = "modExporter"
'Accessible UML

'LICENCE
'This program is free software: you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation, either version 3 of the License, or
'    (at your option) any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program.  If not, see <http://www.gnu.org/licenses/>.

'"Accessible UML" is copyright 2001-2004 Paul Blenkhorn and Gareth Evans, at UMIST,
'and is released by them under the GNU Public Licence Version 3. A copy is available
'at http://www.gnu.org/licenses/gpl.txt
'The code was written by Alasdair King.

'Holds methods and functions for exporting a diagram
'as XMI.
Option Explicit

Private doc As DOMDocument40
Private Const ns As String = "org.omg.xmi.namespace.UML"

Public Sub ExportDiagram(path As String)
'exports current diagram to path
    Set doc = New DOMDocument40
    Dim newAttr As IXMLDOMAttribute
    Dim xmiContent As IXMLDOMNode
    Dim xmiModel As IXMLDOMNode
    
    'load the top node
    doc.async = False
    Call doc.loadXML("<XMI xmlns:UML=""org.omg.xmi.namespace.UML"" version=""1.2"" />")
    Debug.Print doc.namespaces.length
    Debug.Print doc.namespaces(0)
    'populate the attributes
    'version
    Set newAttr = doc.createAttribute("timestamp")
    newAttr.Value = Date
    Call doc.documentElement.attributes.setNamedItem(newAttr)
    'add header
    Call AddXMIHeader
    'add content node
    Set xmiContent = AddEmptyChildNode(doc.documentElement, "XMI.content")
    'add model
    Set xmiModel = AddEmptyChildNode(xmiContent, "UML:Model")
    'finally save to disk
    Call doc.save(path)
End Sub

Private Sub AddXMIHeader()
'add the XMI.header element
    Dim xmiHeader As IXMLDOMNode
    Dim xmiDocumentation As IXMLDOMNode
    
    'add XMI.header node
    Set xmiHeader = AddEmptyChildNode(doc.documentElement, "XMI.header")
    Set xmiDocumentation = AddEmptyChildNode(xmiHeader, "XMI.documentation")
    Call AddChildNode(xmiDocumentation, "XMI.exporter", "AccessibleUML")
    Call AddChildNode(xmiDocumentation, "XMI.exporterVersion", "1.0")
End Sub

Private Function AddEmptyChildNode(parentNode As IXMLDOMNode, nodeName As String) As IXMLDOMNode
'adds a new child node to parentNode with element name nodeName and returns it
    Dim newNode As IXMLDOMNode
    If InStr(1, nodeName, ":") > 0 Then
        'UML:something node: need the namespace
        Set newNode = doc.createNode(NODE_ELEMENT, nodeName, ns)
    Else
        'XMI node: no namespace
        Set newNode = doc.createNode(NODE_ELEMENT, nodeName, Empty)
    End If
    Call parentNode.appendChild(newNode)
    Set AddEmptyChildNode = parentNode.lastChild
End Function

Private Sub AddChildNode(parentNode As IXMLDOMNode, nodeName As String, content As String)
'adds a new child node to parentNode with element name nodeName and content content
    Dim newNode As IXMLDOMNode
    If InStr(1, nodeName, ":") > 0 Then
        'UML:something node: need the namespace
        Set newNode = doc.createNode(NODE_ELEMENT, nodeName, ns)
    Else
        'XMI node: no namespace
        Set newNode = doc.createNode(NODE_ELEMENT, nodeName, Empty)
    End If
    newNode.Text = content
    Call parentNode.appendChild(newNode)
End Sub
