Attribute VB_Name = "modJoystickManager"
Option Explicit

Private lastKey As Integer ' records the last key pressed, so we can ensure that there isn't too fast a repeat
Private buttonDelay As Integer ' slows down rate of response of joystick
Private dxObject As DirectX8
Private diObject As DirectInput8
Public diDevice As DirectInputDevice8
Private diEventHandle As Long
Public diJoystickEffect As DirectInputEffect
Private povTimes(0 To 3) As Long 'holds the last time that the POV button was pressed,
'so the user cannot press the button again within that time.
Private buttonTimes(0 To 8) As Long 'holds the last time a joystick button was pressed,
'so the user cannot press the button again within that time.

Public Enum twist
    None
    Leftturn
    Rightturn
End Enum

Private myJoystickName As String

Public Const COORDINATE_MAX As Long = 65535
Public Const COORDINATE_PARTITION As Long = 65535 / 5 + 1
Private direction_array(5, 5) As Integer

Public x As Long
Public y As Long
Private mTwist As twist
Public direction As Integer 'direction joystick is pointing from 0 to 15 clockwise: 0=nw, 16 = centred
Public Const CENTRED As Integer = 16
Public button As Integer
Public pov As Integer
Private delay As Boolean 'indicates that enough time has passed to process a second keypress.

Public Function joystickName() As String

    On Error Resume Next

    joystickName = myJoystickName
End Function

Public Function twist() As twist
    On Error Resume Next
    twist = mTwist
End Function

Public Sub Initialize()

    On Error Resume Next

    On Error GoTo JoystickError
    Dim diDeviceEnumeration As DirectInputEnumDevices8
    
    Set dxObject = New DirectX8
    Set diObject = dxObject.DirectInputCreate
    Set diDeviceEnumeration = diObject.GetDIDevices(DI8DEVTYPE_JOYSTICK, DIEDFL_ALLDEVICES)
    'Set diDeviceEnumeration = diObject.GetDIDevices(DI8DEVCLASS_GAMECTRL, DIEDFL_ATTACHEDONLY Or DIEDFL_FORCEFEEDBACK)
    
    If diDeviceEnumeration.GetCount = 0 Then
        'there aren't any joysticks attached
        Set diDeviceEnumeration = Nothing
        Set diObject = Nothing
        myJoystickName = Empty
    Else
        'some joysticks
        Set diDevice = diObject.CreateDevice(diDeviceEnumeration.GetItem(1).GetGuidInstance)
        ' tell Windows that reports from it should be processed as a joystick
        Call diDevice.SetCommonDataFormat(DIFORMAT_JOYSTICK)
        'setup an event handling mechanism on frmMain
        diEventHandle = dxObject.CreateEvent(frmMain)
        'Tell the joystick events to be processed
        Call diDevice.SetEventNotification(diEventHandle)
        'Me me me! I'm the only application that can set forces on this joystick!!
        'Otherwise forcefeedback won't work
        Call diDevice.SetCooperativeLevel(frmMain.hWnd, DISCL_EXCLUSIVE Or DISCL_BACKGROUND)
        ' now acquire the device so we can get info from it
        Call diDevice.Acquire
        Call UpdateJoystick
    End If
    direction_array(0, 0) = 0
    direction_array(0, 1) = 15
    direction_array(0, 2) = 14
    direction_array(0, 3) = 13
    direction_array(0, 4) = 12
    direction_array(1, 0) = 1
    direction_array(1, 1) = 16
    direction_array(1, 2) = 16
    direction_array(1, 3) = 16
    direction_array(1, 4) = 11
    direction_array(2, 0) = 2
    direction_array(2, 1) = 16
    direction_array(2, 2) = 16
    direction_array(2, 3) = 16
    direction_array(2, 4) = 10
    direction_array(3, 0) = 3
    direction_array(3, 1) = 16
    direction_array(3, 2) = 16
    direction_array(3, 3) = 16
    direction_array(3, 4) = 9
    direction_array(4, 0) = 4
    direction_array(4, 1) = 5
    direction_array(4, 2) = 6
    direction_array(4, 3) = 7
    direction_array(4, 4) = 8
    
    Exit Sub
JoystickError:
    MsgBox Err.description, vbOKOnly, frmMain.language.GetText("No joystick")
End Sub

Public Sub UpdateJoystick()
    On Error Resume Next
    Dim joystickState As DIJOYSTATE
    'Get the joystick input
    Call diDevice.GetDeviceStateJoystick(joystickState)
    x = joystickState.x
    y = joystickState.y
    direction = direction_array(Int(x / COORDINATE_PARTITION), Int(y / COORDINATE_PARTITION))
    'check for button pressing
    If joystickState.Buttons(0) Then
        If GetTickCount - buttonTimes(0) > 200 Then
            button = 0
            buttonTimes(0) = GetTickCount
        Else
            button = -1
        End If
    ElseIf joystickState.Buttons(1) Then
        If GetTickCount - buttonTimes(1) > 200 Then
            button = 1
            buttonTimes(1) = GetTickCount
        Else
            button = -1
        End If
    ElseIf joystickState.Buttons(2) Then
        If GetTickCount - buttonTimes(2) > 200 Then
            button = 2
            buttonTimes(2) = GetTickCount
        Else
            button = -1
        End If
    ElseIf joystickState.Buttons(3) Then
        If GetTickCount - buttonTimes(3) > 200 Then
            button = 3
            buttonTimes(3) = GetTickCount
        Else
            button = -1
        End If
    ElseIf joystickState.Buttons(4) Then
        If GetTickCount - buttonTimes(4) > 200 Then
            button = 4
            buttonTimes(4) = GetTickCount
        Else
            button = -1
        End If
    ElseIf joystickState.Buttons(5) Then
        If GetTickCount - buttonTimes(5) > 200 Then
            button = 5
            buttonTimes(5) = GetTickCount
        Else
            button = -1
        End If
    ElseIf joystickState.Buttons(6) Then
        If GetTickCount - buttonTimes(6) > 200 Then
            button = 6
            buttonTimes(6) = GetTickCount
        Else
            button = -1
        End If
    ElseIf joystickState.Buttons(7) Then
        If GetTickCount - buttonTimes(7) > 200 Then
            button = 7
            buttonTimes(7) = GetTickCount
        Else
            button = -1
        End If
    ElseIf joystickState.Buttons(8) Then
        If GetTickCount - buttonTimes(8) > 200 Then
            button = 8
            buttonTimes(8) = GetTickCount
        Else
            button = -1
        End If
    Else
        button = -1
    End If
    If joystickState.pov(0) = 0 Then
'        Debug.Print "GTC: " & GetTickCount
'        Debug.Print "Pov: " & povTimes(0)
        If GetTickCount - povTimes(0) > 200 Then
            pov = 0
            povTimes(0) = GetTickCount
        Else
            pov = -1
        End If
    ElseIf joystickState.pov(0) = 18000 Then
        If GetTickCount - povTimes(2) > 200 Then
            pov = 2
            povTimes(2) = GetTickCount
        Else
            pov = -1
        End If
    ElseIf joystickState.pov(0) = 9000 Then
        If GetTickCount - povTimes(1) > 200 Then
            pov = 1
            povTimes(1) = GetTickCount
        Else
            pov = -1
        End If
    ElseIf joystickState.pov(0) = 27000 Then
        If GetTickCount - povTimes(3) > 200 Then
            pov = 3
            povTimes(3) = GetTickCount
        Else
            pov = -1
        End If
    Else
        pov = -1
    End If
    'get turn
    If joystickState.rz < 20000 Then
        mTwist = Leftturn
    ElseIf joystickState.rz > 45535 Then
        mTwist = Rightturn
    Else
        mTwist = None
    End If
    'okay, if this is a different button from the one already pressed, go ahead and press it. Otherwise,
    'only allow the button if we are not delayed
''    If pov * 100 + button <> previousButton Then
''        'new press! We can go ahead with it
''        previousButton = pov * 100 + button
''        delay = True
''    Else
''        'repeat of previous key: we can only proceed if delay is false
''        If delay Then
''            'nope, can't do this yet
''            button = -1
''            pov = -1
''        Else
''            'yep, fine, waited long enough, carry on with the values of button and pov
''        End If
''    End If
End Sub

Public Sub CleanUp()
    On Error Resume Next
    If Not (diDevice Is Nothing) Then
        Call diDevice.Unacquire
    End If
    Set diJoystickEffect = Nothing
End Sub
